!--------------------------------------------------------------------------------------
!				     MODULES EXTERIEURS
!--------------------------------------------------------------------------------------
!     Constantes de dimensionnement a priori (constantes proteges)
	MODULE CONSTANTES	   
	   INTEGER,PARAMETER		:: ND1=4		! Dimension max N 
	   INTEGER,PARAMETER		:: ND2=4		! Dimension max M	
	   INTEGER,PARAMETER		:: NP=ND1*ND2		! N*M
	   INTEGER,PARAMETER		:: NL=50		! TRAN (NL > TRAN+1)
	   INTEGER,PARAMETER		:: NCOR=100		! Nb blocs 
	   INTEGER,PARAMETER		:: NZT=5		! Nb options ZETA
	   INTEGER,PARAMETER		:: NSC=1		! Nb res. scalaire ext
	   INTEGER,PARAMETER		:: NRC=2		! Nb res. scalaire int
	   INTEGER,PARAMETER		:: NTE=5		! Nb res. temps egal
	   INTEGER,PARAMETER		:: NTI=1		! Nb res. temps inegal
	   INTEGER,PARAMETER		:: NTG=3		! Nb res. fct. Green
	   INTEGER,PARAMETER		:: NXC=NSC+NRC  	! Nb res. scalaire tot
	   INTEGER,PARAMETER		:: NCM=NSC+NTE+NTI+NTG	! Nb res. total (YNMES) 
	   INTEGER,PARAMETER		:: NCA=20		! Longueur caracteres
	END MODULE CONSTANTES		
!     ---------------------------------------------------------------------------------	   		     		
!     Fichiers externes (parametres ou variables remanents)
	MODULE FICHIERS
	   USE CONSTANTES,ONLY: NCA
	   CHARACTER(LEN=NCA),PARAMETER		:: FICREF='tampin.dat'	! Source (||)
	   CHARACTER(LEN=NCA),SAVE		:: FICIN		! Entree
	   CHARACTER(LEN=NCA),SAVE		:: FICSED		! Germes	
	   CHARACTER(LEN=NCA),SAVE		:: FICHSIN		! HS archivee
	   CHARACTER(LEN=NCA),SAVE		:: FICHSOUT		! HS sauvegarde
	   CHARACTER(LEN=NCA),SAVE		:: FICGIOUT		! Sortie Green
	   CHARACTER(LEN=NCA),SAVE		:: FICPAOUT		! Sortie para
	   CHARACTER(LEN=NCA),SAVE		:: FICCVOUT		! Sortie covar
	   CHARACTER(LEN=NCA),SAVE		:: FICWEB		! Sortie web
	   CHARACTER(LEN=NCA),SAVE		:: REPWEB		! Repert web
	END MODULE FICHIERS 
!     ---------------------------------------------------------------------------------
!     Parametres de simulation (variables non-remanentes,declaration du type seulement)	
	MODULE PARA_SIM
	   USE CONSTANTES,ONLY: NCM,NZT
	   TYPE PARAMETRE					
	     SEQUENCE
	     REAL(8)				:: BETA,U,MU,REMPLD,TPAR,TPER,TPRM
	     INTEGER				:: TRAN,N,M,MES,NWARM,NMODUL
	     INTEGER				:: NR1,NR2,NR3,NR4
	     INTEGER				:: NBLOC,NIMPRI,NOPT
	     CHARACTER(LEN=1),DIMENSION(NCM)	:: YNMES	
	     CHARACTER(LEN=1),DIMENSION(NZT)	:: ZETA
	   END TYPE PARAMETRE
	END MODULE PARA_SIM
!     ---------------------------------------------------------------------------------
!     Forme factorisee de la matrice de hopping (variables remanentes et constantes)
	MODULE HOPPING
	   USE CONSTANTES,ONLY: NP
	   REAL(8),DIMENSION (NP)		:: DV		! Val. propres de H 
	   REAL(8),DIMENSION (NP,NP)		:: ZZ		! Vect. propres de H
	   REAL(8),DIMENSION (NP,NP)		:: H		! exp(K)
	   REAL(8),DIMENSION (NP,NP)		:: HM		! exp(-K)
	   REAL(8),DIMENSION (NP,NP)		:: H2P		! exp(K/2)
	   REAL(8),DIMENSION (NP,NP)		:: H2M		! exp(-K/2)
	   SAVE							! Remanence des valeurs
	END MODULE HOPPING
!     ---------------------------------------------------------------------------------
!     Localisation des chaines et des colonnes (variables remanentes et constantes) 
	MODULE UTILITAIRE
	   USE CONSTANTES,ONLY: NP,NL,ND1,ND2
	   INTEGER,DIMENSION (NP)		:: CHI		! Localisation chaines
	   INTEGER,DIMENSION (NP)		:: COLI		! Localisation colonnes
	   REAL(8),DIMENSION (NL)		:: W		! Coefficient integrale 
	   REAL(8),DIMENSION (ND1,ND2,NP,NP)	:: COSMAG	! Cosinus precalcules
	   SAVE
	END MODULE UTILITAIRE
!     ---------------------------------------------------------------------------------
!     Identification (variables remanentes et constantes)
	MODULE IDENT
	   USE CONSTANTES,ONLY: NCA
	   CHARACTER(LEN=NCA)			:: NANUM	! Nom fichier sortie
	   CHARACTER(LEN=NCA)			:: PREFIX	! Prefixe sortie 
	   CHARACTER(LEN=NCA)			:: SUFFIX	! Suffixe sortie
	   CHARACTER(LEN=8)			:: DTE		! Date
	   REAL(8)				:: TI		! Heure initiale
	   INTEGER				:: IDNUM	! Germe initial
	   SAVE							! Remanence des valeurs
	END MODULE IDENT
!     ---------------------------------------------------------------------------------
!     Module d'interfacage (cf. WRITEOUT)
	MODULE INTER
	   INTERFACE 
	      SUBROUTINE WRITEOUT(PARA,NU,E,VE,F,VF)
	      USE CONSTANTES,ONLY: ND1,ND2,NL
	      USE PARA_SIM
	      INTEGER,INTENT(IN)					:: NU
	      REAL(8),DIMENSION (ND1,ND2),INTENT(IN),OPTIONAL		:: E,VE
	      REAL(8),DIMENSION (0:NL,ND1,ND2),INTENT(IN),OPTIONAL	:: F,VF
	      TYPE(PARAMETRE),INTENT(IN)				:: PARA
	      END SUBROUTINE WRITEOUT
	   END INTERFACE
	END MODULE INTER
!--------------------------------------------------------------------------------------
!				PROGRAMME PRINCIPAL
!--------------------------------------------------------------------------------------
	PROGRAM HUBB
!--------------------------------------------------------------------------------------
! Nom	     :	HUBB	[MAIN]							0
!	
! Version    : 	2_0.0 Beta (20 aout 1997) (version anterieure: hubb1_0)
!
! Fonction   :	Simulation Monte Carlo du modele de Hubbard basee sur la
!		methode du determinant (BSS)
!
! Auteur     : 	H.Touchette selon la version trnb de A.-M.Tremblay
!
! Adresse    : 	Centre de recherche en physique du solide (CRPS)
!		Departement de physique, Universite de Sherbrooke
!		Sherbrooke, Quebec, Canada, J1K 2R1
!		hugo.touchette@physique.usherb.ca  (H. Touchette)
!		tremblay@physique.usherb.ca	   (A.-M. Tremblay)
!
! References :  [1] Programme trnb3, prog10#, et references citees	
!		[2] Notes personnelles de Hugo Touchette, dept. de physique,
!		    Universite de Sherbrooke, 1996
!		[3] H.Touchette, Rapports techniques des projets MCQ, CRPS, 1996-1997  
!		    http://www.physique.usherb.ca/~tremblay/hugo/
!
! Notes: 	
! - Version Fortran90 de trnb ou prog10 (compilation xlf90)
! - Traitement du cas U > 0 et U < 0
! - Tient compte de la symetrie GUP-GDWN pour U < 0
! - Moyennage optionnel des fcts de Green a U > 0 (MESURE) 
! - Nouvelles options de calcul ZETA(1 a 5)
! - Stabilisation du calcul des fonctions de Green a temps egal et inegal
! - Nouveau generateur de nombres pseudo-aleatoires GGUBFS (Numerical Recipes)
! - Remplacement des COMMON par des MODULE 	
! - Nouveau MODULE d'accumulation pour le calcul des statistiques
! - Changements pour la version parallele notes (||)
! - Nouveau format de sortie et automatisation de l'archivage sur le web
! - Sortie optionnelle des configuration HS. Forme: NM colonnes par TRAN lignes
! - Calcul optionnel de la matrice de covariance (COVARIANCE)
! - Calcul optionnel de la self-energie (SELF)
! 		
!--------------------------------------------------------------------------------------
	USE CONSTANTES,ONLY: NP,NL
	USE FICHIERS
	USE PARA_SIM
	USE IDENT
	IMPLICIT NONE
!     Variables scalaires
	REAL(8)				:: SIG
	REAL(8)				:: DTAU,CTE2
	REAL(8)				:: VU,VD,VARUP,VARDWN
	REAL(8)				:: RUP,RDWN,R,P,ALEA
	REAL(8)				:: DETU,DETD,DETUN,DETDN
	INTEGER				:: NM,MESC,NCHN,NHS,I
	INTEGER				:: NCG,NST
	INTEGER				:: NSWEEP,L,NS		! Boucles principales
	INTEGER(SELECTED_INT_KIND(9))	:: DSEED,DSEEDI		! Germes	
!     Fonctions	
	CHARACTER(8)			:: DATE			! Date mm/dd/yy		
	REAL(8) 			:: GGUBFS		! Gen nb aleatoires
	INTEGER				:: MCLOCK		! Horloge CPU
!     Vecteurs, matrices	
	REAL(8),DIMENSION (2)		:: COMPT		! Compteurs	
	REAL(8),DIMENSION (NP,NL)	:: VUP,VDWN		! Interaction 	
	REAL(8),DIMENSION (NP,NP)	:: GUP,GDWN		! Fonctions de Green
	REAL(8),DIMENSION (NP,NP)	:: GUPT,GDWNT		! Fonctions de Green
	INTEGER,DIMENSION (NP,NL)	:: SPIN			! Spins 1 champ HS
!     Type derive
	TYPE(PARAMETRE)			:: PARA			! Parametres simulation
!     ---------------------------------------------------------------------------------
	TI=MCLOCK()						! Temps depart
	DTE=DATE()						! Date depart
!
!	Lecture noms de fichiers. Pour la version parallele UNIT=13 -> UNIT='*' (||)
	OPEN(UNIT=13,FILE=FICREF,STATUS='OLD',ERR=111)
	REWIND(UNIT=13)
	READ(13,*) SUFFIX
	READ(13,*) PREFIX
	READ(13,*) FICSED
	READ(13,*) FICIN
	READ(13,*) FICHSIN
	READ(13,*) FICWEB
	READ(13,*) REPWEB
	CLOSE(UNIT=13)
!
!	Initialisation du germe
	OPEN(UNIT=4,FILE=FICSED,STATUS='OLD',ERR=111)
	REWIND(UNIT=4)
	READ(4,*) DSEED
	DSEEDI=DSEED	
	CLOSE(UNIT=4)
!     ----------------------------BLOC D'INITIALISATION--------------------------------
	CALL ENTREE(PARA)					! Parametres (module)
	NM=PARA%N*PARA%M					! Nb de sites reels
	CALL IDENTIFICATION(PARA,DSEEDI)			! Id fichiers
	CALL INIT_LOC(PARA%N,PARA%M)				! Vec. localisation	
	CALL DEFCOSMAG(PARA%N,PARA%M)				! Cosinus pre-calcules
	CALL INITW(PARA%TRAN)					! Coef. d'integration
	CALL INITH3(NM,PARA)					! H dense
	CALL INITV(PARA,DSEED,VU,VD,SPIN,VUP,VDWN)		! Matrices interaction 		
!
!	Ouverture des fichiers d'ecriture 
	OPEN(UNIT=2,FILE=FICPAOUT)
	OPEN(UNIT=10,FILE=FICHSOUT)
	OPEN(UNIT=11,FILE=FICGIOUT)
	REWIND(UNIT=2)
	REWIND(UNIT=10)
	REWIND(UNIT=11)
!     ---------------------------------SIMULATION--------------------------------------
!	Initialisation des constantes de depart
	SIG=1.D0						! Signe initial
	MESC=0							! Mesures effectuees
	COMPT(1:2)=0.D0						! Flips acceptes
	DTAU=PARA%BETA/DBLE(PARA%TRAN)
	CTE2=2*DLOG(VU)						! U < 0	(pour R)
!
	! Boucles principales
  	DO 100 NSWEEP=1,PARA%MES+PARA%NWARM			! Mesures+rechauf
	   DO 120 L=1,PARA%TRAN					! Tranches temps
!	         
	      ! Calcul des fcts de Green a temps egal pour la tranche L
	      ! Calcul complet a tous les NMODUL
	      NCHN=MOD(L,PARA%NMODUL)
	      IF(NCHN.EQ.1.OR.L.EQ.1) THEN			! Calcul complet
	         CALL GCALS(NM,L,PARA,VUP,GUP,DETU)
	         SELECT CASE(PARA%U>0)				! Symetrie GUP,GDWN
	            CASE(.TRUE.)
	               CALL GCALS(NM,L,PARA,VDWN,GDWN,DETD)
	            CASE(.FALSE.)
	               CALL MAC10(NP,NM,GUP,GDWN)
	               DETD=DETU
	         END SELECT
	         IF(NSWEEP.GT.PARA%NWARM.AND.L.EQ.1) THEN
		    MESC=MESC+1
	            CALL MESURE(GUP,GDWN,VUP,VDWN,MESC,COMPT,SIG,PARA)
	            IF(MESC.EQ.PARA%MES) GOTO 101				! Fin
	         ENDIF
	      ELSE								! Wrap
	         CALL MAC4251(NM,1,GUP,VUP(1,L-1))
	         SELECT CASE(PARA%U>0)				! Symetrie GUP,GDWN
	            CASE(.TRUE.);  CALL MAC4251(NM,1,GDWN,VDWN(1,L-1)) 
	            CASE(.FALSE.); CALL MAC10(NP,NM,GUP,GDWN)
	         END SELECT	
	      ENDIF
!
		 ! Mouvements globaux 
		 IF((PARA%ZETA(4).EQ.'Y'.OR.PARA%ZETA(4).EQ.'y').AND.L.EQ.1) THEN
	   	    NCG=INT(GGUBFS(DSEED)*NM)+1	
	   	    NST=SUM(SPIN(NCG,1:PARA%TRAN))		
		    CALL GLOBAL_MOVE(PARA%TRAN,VU,VD,PARA%U,SPIN,VUP,VDWN,NCG)
		    CALL GCALS(NM,L,PARA,VUP,GUPT,DETUN)
		    SELECT CASE(PARA%U>0)
		       CASE(.TRUE.)
		          CALL GCALS(NM,L,PARA,VDWN,GDWNT,DETDN)
		       CASE(.FALSE.)
		          CALL MAC10(NP,NM,GUPT,GDWNT)
		          DETDN=DETUN
		    END SELECT
		    RUP=DETUN/DETU 
		    RDWN=DETDN/DETD 
                    SELECT CASE(PARA%U>0)
                       CASE(.TRUE.);  R=RUP*RDWN				! U > 0
                       CASE(.FALSE.); R=RUP*RDWN*DEXP(CTE2*DBLE(NST))		! U < 0 
                    END SELECT
		    P=DABS(R)/(DABS(R)+1.D0)				
		    ALEA=GGUBFS(DSEED)
		    IF(ALEA.LE.P) THEN
		       COMPT(2)=COMPT(2)+1.D0
		       SIG=SIG*DSIGN(1.D0,R)				
		       CALL MAC10(NP,NM,GUPT,GUP)
		       CALL MAC10(NP,NM,GDWNT,GDWN)
		    ELSE
		       CALL GLOBAL_MOVE(PARA%TRAN,VU,VD,PARA%U,SPIN,VUP,VDWN,NCG)
		    ENDIF
		 ENDIF
!
	      ! Balayage des sites d'une tranche de temps
	      DO 130 NS=1,NM
!
		 ! Definition des constantes selon U et la valeur du champ HS	            
	         IF(PARA%U.GT.0) THEN
                    IF(SPIN(NS,L).EQ.1) THEN			
		       VARUP=VD/VU-1.D0				! SPIN = 1,U > 0
                       VARDWN=VU/VD-1.D0
		    ELSE
		       VARUP=VU/VD-1.D0				! SPIN =-1,U > 0
                       VARDWN=VD/VU-1.D0
		    ENDIF
                 ELSE
                    IF(SPIN(NS,L).EQ.1) THEN
                       VARUP=VD/VU-1.D0				! SPIN = 1,U < 0
                       VARDWN=VD/VU-1.D0
                    ELSE
                       VARUP=VU/VD-1.D0				! SPIN =-1,U < 0
                       VARDWN=VU/VD-1.D0
                    ENDIF
                 ENDIF
!
                 ! Calcul des determinants pour la probabilite de flip 
                 RUP=1.D0+(1.D0-GUP(NS,NS))*VARUP
                 RDWN=1.D0+(1.D0-GDWN(NS,NS))*VARDWN
                 IF(PARA%U.GT.0) THEN
        	    R=RUP*RDWN					! U > 0
		 ELSE
		    R=RUP*RDWN*DEXP(CTE2*DBLE(SPIN(NS,L)))	! U < 0
		 END IF
		 P=DABS(R)/(DABS(R)+1.D0)			! Probabilite
!
		 ! Changement de la valeur du champ HS selon la valeur de ALEA
		 ALEA=GGUBFS(DSEED)
		 IF(ALEA.LE.P) THEN				
	            COMPT(1)=COMPT(1)+1.D0			! Nb spins flippes
	            SIG=SIG*DSIGN(1.D0,R)			! Signe du det.
	            IF(PARA%U.GT.0) THEN
                       IF(SPIN(NS,L).EQ.1) THEN
		          VUP(NS,L)=VD				! SPIN = 1,U > 0
                          VDWN(NS,L)=VU
                          SPIN(NS,L)=-1
		       ELSE
		          VUP(NS,L)=VU				! SPIN =-1,U > 0
                          VDWN(NS,L)=VD
                          SPIN(NS,L)=1
                       ENDIF
                    ELSE
                       IF(SPIN(NS,L).EQ.1) THEN
	 	          VUP(NS,L)=VD				! SPIN = 1,U < 0
                          VDWN(NS,L)=VD
                          SPIN(NS,L)=-1
		       ELSE
		          VUP(NS,L)=VU				! SPIN =-1,U < 0
                          VDWN(NS,L)=VU
                          SPIN(NS,L)=1
		       ENDIF
                    ENDIF
                    CALL UPDTG(NP,NM,NS,GUP,VARUP)		! Mise a jour de G
                    CALL UPDTG(NP,NM,NS,GDWN,VARDWN)
                 ENDIF		    
130           END DO					
120	   END DO 
!
	   ! Sauvegarde de la configuration HS a tous les NIMPRI
	   NHS=MOD(NSWEEP,PARA%NIMPRI)
	   IF(NHS.EQ.0.AND.(PARA%ZETA(3).EQ.'Y'.OR.PARA%ZETA(3).EQ.'y')) THEN
	      WRITE(10,FMT='(I8,A)') DSEED,' Germe'
	      WRITE(10,FMT='(I8,A)') NSWEEP+1,' Mesures effectuees'
	      WRITE(10,109) (SPIN(:,I),I=1,PARA%TRAN)
	   ENDIF
100	END DO
!     -----------------------------FIN DE LA SIMULATION--------------------------------
101	CONTINUE
!	Remplacement du germe
	OPEN(UNIT=4,FILE=FICSED,STATUS='OLD')
	REWIND(UNIT=4)
	WRITE(4,*) DSEED
	CLOSE(UNIT=4)
!
!	Fermeture des fichiers 
	CLOSE(UNIT=2)	
	CLOSE(UNIT=10)
	CLOSE(UNIT=11)
109	FORMAT(<NM>(I4))
	STOP
111	WRITE(*,*) '--- Erreur : Fichier inexistant (MAIN) ---'
	STOP
!
	END PROGRAM HUBB
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
!				  PROCEDURES EXTERNES
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
! Nom		: ACCUMULATION_STATISTIQUE	[MODULE]			1
! Fonction	: Accumulation des resultats des mesures et controle de la sortie
! Auteurs	: H.Touchette
! Revision	: 17 novembre 1996
! References	: [1] H.Touchette, Algorithme MCQ: version Fortran 90, rapport 
!		      technique du CRPS, automne 1996.
! Notes		: - Les variables d'accumulation sont rangees dans un MODULE et doivent 
!		    etre explicitement declarees avec l'attribut SAVE pour qu'elles
!		    demeurent remanentes.
!		  - Pour la signification des etages (6) d'accumulation, cf. MESURE2
!		  - NW3 controle la remise a zera de l'etage 2 a tous les blocs
!
! Dependances	: 		       |- Declarations des variables d'accumulation 
!		  MODULE accumulation -|- ACCUMULATOR [SUB] 
!--------------------------------------------------------------------------------------	            
	MODULE ACCUMULATION_STATISTIQUE
!	   ----------------------------------------------------------------------------          
!          Variables d'accumulation pour les mesures (variables remanentes)
	   USE CONSTANTES,ONLY: ND1,ND2,NL,NTE,NTI,NTG,NXC,NCOR
	   REAL(8)					:: SIGMOY,SBLOC,C1
	   REAL(8),DIMENSION (NCOR)			:: AUTOCORR	! Correlation 
	   REAL(8),DIMENSION (6,0:NXC)			:: XM		! Res. courants
	   REAL(8),DIMENSION (6,NTE,ND1,ND2)		:: XSTRU	! Fact. struct
	   REAL(8),DIMENSION (6,NTI,0:NL,ND1,ND2)	:: XSTRUI	! Fact. structI
	   REAL(8),DIMENSION (6,NTG,0:NL,ND1,ND2)	:: XGIUP	! Inegal G
	   REAL(8),DIMENSION (6,NTG,0:NL,ND1,ND2)	:: XGIDN	! Inegal G
	   REAL(8),DIMENSION (NCOR,0:NL,ND1,ND2)	:: COVGU	! Covariance
	   REAL(8),DIMENSION (0:NL,0:NL,ND1,ND2)	:: COVMU	! Covariance	
	   SAVE							! Remanence des valeurs
!	   ----------------------------------------------------------------------------
	CONTAINS						
!	   ----------------------------------------------------------------------------
	   SUBROUTINE ACCUMUL(COMPT,MESC,SIG,PARA,DMU,XSTP,FACST,FACSTI,FGIUP,FGIDN)
	   USE PARA_SIM
	   IMPLICIT NONE
	   REAL(8)						:: DERIV,XMUPROP 
	   REAL(8)						:: FLUCT,ENERG
	   INTEGER						:: NW1,NW2	
	   INTEGER						:: I,NM,NCOMES	
	   REAL(8),INTENT(IN)					:: DMU,SIG
	   INTEGER,INTENT(IN)					:: MESC	
	   REAL(8),DIMENSION (2)				:: COMPT,XCOMP		
	   REAL(8),DIMENSION (0:NXC),INTENT(IN)			:: XSTP		
	   REAL(8),DIMENSION (NTE,ND1,ND2),INTENT(IN)		:: FACST
	   REAL(8),DIMENSION (NTI,0:NL,ND1,ND2),INTENT(IN)	:: FACSTI
	   REAL(8),DIMENSION (NTG,0:NL,ND1,ND2),INTENT(IN)	:: FGIUP,FGIDN	
	   TYPE(PARAMETRE),INTENT(IN)				:: PARA
!
	   NM=PARA%N*PARA%M
!
!	   Initialisation des variables a la premiere mesure
	   IF(MESC.EQ.1) THEN
	      SIGMOY=0.D0
	      SBLOC=0.D0 
	      C1=0.D0
	      XM(1:6,0:NXC)=0.D0					
	      XSTRU(1:6,1:NTE,1:PARA%N,1:PARA%M)=0.D0			 
	      XGIUP(1:6,1:NTG,0:PARA%TRAN,1:PARA%N,1:PARA%M)=0.D0	
	      XGIDN(1:6,1:NTG,0:PARA%TRAN,1:PARA%N,1:PARA%M)=0.D0	
	      XSTRUI(1:6,1:NTI,0:PARA%TRAN,1:PARA%N,1:PARA%M)=0.D0
	      AUTOCORR(1:NCOR)=0.D0
	      COVGU(1:NCOR,0:PARA%TRAN,1:PARA%N,1:PARA%M)=0.D0
	   ENDIF
!	
!	   Incrementation du signe moyen et SBLOC
	   SIGMOY=SIGMOY+SIG
	   SBLOC=SBLOC+SIG
!
!	   Initialisation des variables pour les statistiques 
	   NW1=0
	   NW2=0
	   IF(MOD(MESC,PARA%NBLOC).EQ.0) THEN			! Calcul par blocs
	      NW1=1
	      C1=C1+1.D0
	   ENDIF
	   NCOMES=MOD(MESC,PARA%NIMPRI)
	   IF(NCOMES.EQ.0.OR.MESC.EQ.PARA%MES) NW2=1		! Calcul impression
!
!	   Choix des statistiques
!	   ----------------------------------------------------------------------------
!	   Statistiques sur les resultats courants (non-optionnel)		! 1
	   CALL MESURE2(NXC,0,NXC,SIG,SBLOC,SIGMOY,C1,NW1,NW2,0,XSTP,XM)
!
!	   Statistique sur les quantites a temps egal (optionnel)	
	   IF(PARA%YNMES(2).EQ.'Y'.OR.PARA%YNMES(2).EQ.'y') THEN		! 2			   
	      CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
		          & NW1,NW2,1,FACST(1,:,:),XSTRU(:,1,:,:))
	   ENDIF
	   IF(PARA%YNMES(3).EQ.'Y'.OR.PARA%YNMES(3).EQ.'y') THEN		! 3		 
	      CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
		          & NW1,NW2,1,FACST(2,:,:),XSTRU(:,2,:,:))
	   ENDIF
	   IF(PARA%YNMES(4).EQ.'Y'.OR.PARA%YNMES(4).EQ.'y') THEN		! 4
	      CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
		          & NW1,NW2,1,FACST(3,:,:),XSTRU(:,3,:,:))
	   ENDIF
	   IF(PARA%YNMES(5).EQ.'Y'.OR.PARA%YNMES(5).EQ.'y') THEN		! 5	   
	      CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
		          & NW1,NW2,1,FACST(4,:,:),XSTRU(:,4,:,:))
	   ENDIF
	   IF(PARA%YNMES(6).EQ.'Y'.OR.PARA%YNMES(6).EQ.'y') THEN		! 6	   
	      CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
		          & NW1,NW2,1,FACST(5,:,:),XSTRU(:,5,:,:))
	   ENDIF
!
!	   Statistique sur les quantites a temps inegal (optionnel)
	   IF(PARA%YNMES(7).EQ.'Y'.OR.PARA%YNMES(7).EQ.'y') THEN		! 7
	      DO I=0,PARA%TRAN
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      	             & NW1,NW2,1,FACSTI(1,I,:,:),XSTRUI(:,1,I,:,:))
	      END DO
	   ENDIF
!	
!	   Statistiques sur les fcts de Green et les quantites derivees
	   IF(PARA%YNMES(8).EQ.'Y'.OR.PARA%YNMES(8).EQ.'y') THEN		! 8
	      DO I=0,PARA%TRAN
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      		     & NW1,NW2,0,FGIUP(1,I,:,:),XGIUP(:,1,I,:,:))
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      	             & NW1,NW2,1,FGIDN(1,I,:,:),XGIDN(:,1,I,:,:))
	      END DO
	   ENDIF
	   IF(PARA%YNMES(9).EQ.'Y'.OR.PARA%YNMES(9).EQ.'y') THEN		! 9
	      DO I=0,PARA%TRAN
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      		     & NW1,NW2,1,FGIUP(2,I,:,:),XGIUP(:,2,I,:,:))
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      	             & NW1,NW2,1,FGIDN(2,I,:,:),XGIDN(:,2,I,:,:))
	      END DO
	   ENDIF
	   IF(PARA%YNMES(10).EQ.'Y'.OR.PARA%YNMES(10).EQ.'y') THEN		! 10
	      DO I=0,PARA%TRAN
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      		     & NW1,NW2,1,FGIUP(3,I,:,:),XGIUP(:,3,I,:,:))
	         CALL MESURE3(ND1,ND2,1,PARA%N,PARA%M,SIG,SBLOC,SIGMOY,C1,&
	      	             & NW1,NW2,1,FGIDN(3,I,:,:),XGIDN(:,3,I,:,:))
	      END DO
	   ENDIF	
!
!	   Controle des blocs
	   IF(MOD(MESC,PARA%NBLOC).EQ.0) THEN
	      AUTOCORR(C1)=XM(2,2)/SBLOC
	      COVGU(C1,0:PARA%TRAN,1:PARA%N,1:PARA%M)=&
	      & XGIUP(2,1,0:PARA%TRAN,1:PARA%N,1:PARA%M)/SBLOC
	      XM(2,0:NXC)=0.D0					! Remise a 0 de l'etage 2
	      XGIUP(2,1,0:PARA%TRAN,1:PARA%N,1:PARA%M)=0.D0	! Remise a 0 de l'etage 2
	      SBLOC=0.D0
	   ENDIF
!
!	   Controle de la sortie	
	   NCOMES=MOD(MESC,PARA%NIMPRI)
	   IF(NCOMES.EQ.0.OR.MESC.EQ.PARA%MES) THEN
!
	      ! Calcul mu propose et fluct. nb particules
	      DERIV=-DMU/SIGMOY-PARA%BETA*NM*XM(5,0)**2			
	      XMUPROP=PARA%MU+(PARA%REMPLD-XM(5,0))/DERIV		
	      XCOMP(1)=COMPT(1)/(MESC+PARA%NWARM)/(NM*PARA%TRAN)
	      XCOMP(2)=COMPT(2)/(MESC+PARA%NWARM)		
	      FLUCT=DSQRT(ABS(DERIV/(PARA%BETA*NM)))			
!
	      ! Calcul de l'energie des electrons libres
	      CALL ENE_TOT(PARA,ENERG)
!
	      ! Sortie des resultats
	      CALL SORTIE1(PARA,MESC,XCOMP,SIGMOY,FLUCT,XMUPROP,ENERG,XM(5,:),XM(6,:))
	      CALL SORTIE2(PARA,XSTRU(5,:,:,:),XSTRU(6,:,:,:),&
	      		  &XSTRUI(5,:,:,:,:),XSTRUI(6,:,:,:,:))	
	      CALL SORTIE3(PARA,MESC,XGIUP(5,:,:,:,:),XGIDN(5,:,:,:,:),&
	         	  &XGIUP(6,:,:,:,:),XGIDN(6,:,:,:,:))
!
	      IF(MESC.EQ.PARA%MES) THEN
	         IF(PARA%ZETA(1).EQ.'y'.OR.PARA%ZETA(1).EQ.'Y') THEN
	            CALL AUTO_CORRELATION(C1,AUTOCORR)
	         ENDIF
	         IF(PARA%ZETA(5).EQ.'y'.OR.PARA%ZETA(5).EQ.'Y') THEN 
	            CALL COVARIANCE(PARA,C1,COVGU,XGIUP(5,1,:,:,:),COVMU)
	         ENDIF
	         CALL SORTIEC(PARA,MESC,C1,AUTOCORR,COVMU)
	         CALL SORTIE4(PARA,XMUPROP,XM(5,0))
	      ENDIF
	   ENDIF
!	
	   END SUBROUTINE ACCUMUL
!	   ----------------------------------------------------------------------------
	END MODULE ACCUMULATION_STATISTIQUE
!--------------------------------------------------------------------------------------
! Nom		: MESURE 	[SUB]						2
! Fonction	: Appel des routines de mesure 
! Auteurs	: AM.Tremblay,S.Caille,H.Touchette
! Revision	: 20 fevrier 1996 (Touchette)
! Notes		: - L'extension de cette routine pour de nouvelles mesures est decrite
!		    dans les rapports des projets MCQ,H.Touchette,CPRS,Automne 1996
!		  - FGIUP(2,:,:,:)=Re{G(iwn,Qx,Qy)}
!		  - FGIDN(2,:,:,:)=Im{G(iwn,Qx,Qy)}
!		  - FGIUP(3,:,:,:)=Re{self-energie}
!		  - FGIDN(3,:,:,:)=Im{self-energie}
!		  - Le moyennage des fcts de Green up et down a U > 0 est optionnel.
!		    Enlevez les commentaires ci-dessous pour activer cette option.
!--------------------------------------------------------------------------------------	
	SUBROUTINE MESURE(GUP,GDN,VUP,VDN,MESC,COMPT,SIG,PARA)
	USE CONSTANTES,ONLY: NP,ND1,ND2,NL,NTE,NTI,NTG,NXC
	USE PARA_SIM
	USE ACCUMULATION_STATISTIQUE
	IMPLICIT NONE
	INTEGER						:: NM		! Var locale 
	REAL(8),SAVE					:: DMU		! Remanent
	REAL(8),INTENT(IN)				:: SIG
	INTEGER,INTENT(IN)				:: MESC
	REAL(8),DIMENSION (2)				:: COMPT 
	REAL(8),DIMENSION (NP,NP),INTENT(IN)		:: GUP,GDN
	REAL(8),DIMENSION (NP,NL),INTENT(IN)		:: VUP,VDN
	REAL(8),DIMENSION (0:NXC)			:: XSTP		! Res. courants
	REAL(8),DIMENSION (NTE,ND1,ND2)			:: FACST	! Fact. struct
	REAL(8),DIMENSION (NTI,0:NL,ND1,ND2)		:: FACSTI	! Fact. structI	
	REAL(8),DIMENSION (0:NL,NP,NP)			:: GIUP,GIDN	! Green
	REAL(8),DIMENSION (NTG,0:NL,ND1,ND2)		:: FGIUP,FGIDN ! Green,Self
	TYPE(PARAMETRE),INTENT(IN)			:: PARA	
!
	NM=PARA%N*PARA%M
	IF(MESC.EQ.1) DMU=0.D0				
!
!	Calcul des fonctions de Green a temps inegal 
	CALL GICALS(NM,PARA,GUP,VUP,GIUP)				! Green inegal
	CALL GICALS(NM,PARA,GDN,VDN,GIDN)				! Green inegal
!
!	Moyennage des fcts de Green up et down pour U > 0
!	GIUP(0:PARA%TRAN,1:NM,1:NM)=&
!	& (GIUP(0:PARA%TRAN,1:NM,1:NM)+GIDN(0:PARA%TRAN,1:NM,1:NM))/2
!	GIDN(0:PARA%TRAN,1:NM,1:NM)=GIUP(0:PARA%TRAN,1:NM,1:NM)	
!
! 	Mesures : choix et appel des mesures a effectuer		
!	-------------------------------------------------------------------------------
	IF(PARA%YNMES(1).EQ.'Y'.OR.PARA%YNMES(1).EQ.'y') THEN			! 1
	   CALL DSITE(PARA,GIUP(0,:,:),GIDN(0,:,:),XSTP(1),XSTP(2))
	ENDIF
	IF(PARA%YNMES(2).EQ.'Y'.OR.PARA%YNMES(2).EQ.'y') THEN			! 2
	   CALL MAGST(PARA%N,PARA%M,GIUP(0,:,:),GIDN(0,:,:),FACST(1,:,:))
	ENDIF
	IF(PARA%YNMES(3).EQ.'Y'.OR.PARA%YNMES(3).EQ.'y') THEN			! 3
	   CALL DENSITE(PARA%N,PARA%M,GIUP(0,:,:),GIDN(0,:,:),FACST(2,:,:))
	ENDIF
	IF(PARA%YNMES(4).EQ.'Y'.OR.PARA%YNMES(4).EQ.'y') THEN			! 4
	   CALL RHOE(PARA%N,PARA%M,GIUP(0,:,:),GIDN(0,:,:),FACST(3,:,:))
	ENDIF
	IF(PARA%YNMES(5).EQ.'Y'.OR.PARA%YNMES(5).EQ.'y') THEN			! 5
	   CALL PSCOR(PARA%N,PARA%M,GIUP(0,:,:),GIDN(0,:,:),FACST(4,:,:))
	ENDIF
	IF(PARA%YNMES(7).EQ.'Y'.OR.PARA%YNMES(7).EQ.'y') THEN			! 6+7
	   CALL SUSCEPI(PARA,GIUP,GIDN,FACSTI(1,:,:,:),FACST(5,:,:))	
	ENDIF
	IF(PARA%YNMES(8).EQ.'Y'.OR.PARA%YNMES(8).EQ.'y') THEN			! 8
	   CALL FTQ(PARA%N,PARA%M,PARA%TRAN,GIUP,FGIUP(1,:,:,:))	
	   CALL FTQ(PARA%N,PARA%M,PARA%TRAN,GIDN,FGIDN(1,:,:,:))	
	ENDIF
	IF(PARA%YNMES(9).EQ.'Y'.OR.PARA%YNMES(9).EQ.'y') THEN			! 9
	   CALL MATSUBARA(PARA%N,PARA%M,PARA%TRAN,PARA%BETA,&
	   		 &0,FGIUP(1,:,:,:),FGIUP(2,:,:,:))	
	   CALL MATSUBARA(PARA%N,PARA%M,PARA%TRAN,PARA%BETA,&
	   		 &1,FGIUP(1,:,:,:),FGIDN(2,:,:,:))	
	ENDIF
	IF(PARA%YNMES(10).EQ.'Y'.OR.PARA%YNMES(10).EQ.'y') THEN			! 10
	   CALL SELF(PARA,FGIUP(2,:,:,:),FGIDN(2,:,:,:),FGIUP(3,:,:,:),FGIDN(3,:,:,:))
	ENDIF
!
!	Calcul du nombre moyen d'occupation et mu propose
	CALL OCC_REMP(NP,NM,SIG,GIUP(0,:,:),GIDN(0,:,:),DMU,XSTP(0),PARA%BETA) 
!
!	Exportation des resultats de mesures pour l'accumulation 
	CALL ACCUMUL(COMPT,MESC,SIG,PARA,DMU,XSTP,FACST,FACSTI,FGIUP,FGIDN)
			
!
	END SUBROUTINE MESURE
!--------------------------------------------------------------------------------------
! Nom		: MESURE2 	[SUB]						3-(1)
! Fonction	: Calcul des statistiques sur les mesures (resultats courants)
! Auteurs	: S.Caille,AM.Tremblay,H.Touchette
! Revision	: Decembre 1990
! Reference 	: [1] H.Touchette, Algorithme MCQ: version Fortran 90, rapport 
!		      technique du CRPS, automne 96.
! Notes		: 
! Variables	: XME(i,j) -> i: etage d'accumulation
!	      		      j: indice du vecteur contenant les resultats
!    
! Etages 		Signification
!   1		Accumulation en vue de la moyenne totale
!   2		Accumulation sur un bloc en vue de la variance
!   3		Moyenne des blocs
!   4		Moyenne des blocs au carre	
!   5		Moyenne totale
!   6		Variance et ecart-type
!
! SIG 	  	: valeur du signe
! SBLOC   	: nombre de mesures par bloc
! SIGMOY  	: valeur moyenne du signe
! C1	  	: nombre de blocs
! NW1,NW2,NW3 	: indicateurs de conditions:
! Si NW1=1, Le calcul est fait pour le nombre de blocs; le compteur est mis a zero.
! Si NW2=1, Le calcul est fait pour en prevision de l'impression.
! Si NW3=1, L'etage 2 n'est pas remis a zero dans la routine
!--------------------------------------------------------------------------------------
	SUBROUTINE MESURE2(NP,N1,M1,SIG,SBLOC,SIGMOY,C1,NW1,NW2,NW3,VME,XME)
	IMPLICIT NONE
	INTEGER					:: I			! Var. locale
	REAL(8),INTENT(IN)			:: SBLOC,SIGMOY,C1,SIG
	INTEGER,INTENT(IN)			:: NP,N1,M1,NW1,NW2,NW3
	REAL(8),DIMENSION (N1:NP),INTENT(IN) 	:: VME
	REAL(8),DIMENSION(6,N1:NP),INTENT(INOUT):: XME	
!
	DO I=N1,M1
	   XME(1,I)=XME(1,I)+VME(I)*SIG
	   XME(2,I)=XME(2,I)+VME(I)*SIG
	END DO
	IF(NW1.EQ.1) THEN
	   DO I=N1,M1
	      XME(3,I)=XME(3,I)+XME(2,I)/SBLOC
	      XME(4,I)=XME(4,I)+(XME(2,I)/SBLOC)**2
	   END DO
	ENDIF
	IF(NW1.EQ.1.AND.NW3.EQ.1) XME(2,N1:M1)=0.D0
	IF(NW2.EQ.1) THEN
	   DO I=N1,M1
	      XME(5,I)=XME(1,I)/SIGMOY
	      XME(6,I)=XME(4,I)/C1-(XME(3,I)/C1)**2
	      XME(6,I)=DSQRT(XME(6,I)/C1)
	   END DO
	ENDIF
	END SUBROUTINE MESURE2
!--------------------------------------------------------------------------------------
! Nom		: MESURE3 	[SUB]						3-(2)
! Fonction	: Calcul des statistiques sur les facteurs de structures
! Auteurs	: H.Touchette
! Revision	: 7 novembre 1996
! Notes		: Voir commentaires de MESURE2 
!--------------------------------------------------------------------------------------    
	SUBROUTINE MESURE3(NP1,NP2,N1,N,M,SIG,SBLOC,SIGMOY,C1,NW1,NW2,NW3,VME,XME)
	IMPLICIT NONE
	INTEGER						:: I,J	! Var. locales 
	REAL(8),INTENT(IN)				:: SBLOC,SIGMOY,C1,SIG
	INTEGER,INTENT(IN)				:: NP1,NP2,N1,N,M
	INTEGER,INTENT(IN)				:: NW1,NW2,NW3
	REAL(8),DIMENSION (N1:NP1,N1:NP2),INTENT(IN)	:: VME
	REAL(8),DIMENSION(6,N1:NP1,N1:NP2),INTENT(INOUT):: XME
!
	DO J=N1,M
	   DO I=N1,N
	      XME(1,I,J)=XME(1,I,J)+VME(I,J)*SIG
	      XME(2,I,J)=XME(2,I,J)+VME(I,J)*SIG
	   END DO
	END DO	
	IF(NW1.EQ.1) THEN
	   DO J=N1,M
	      DO I=N1,N
	         XME(3,I,J)=XME(3,I,J)+XME(2,I,J)/SBLOC
	         XME(4,I,J)=XME(4,I,J)+(XME(2,I,J)/SBLOC)**2
	      END DO
	   END DO	
	ENDIF
	IF(NW1.EQ.1.AND.NW3.EQ.1) XME(2,N1:N,N1:M)=0.D0
	IF(NW2.EQ.1) THEN
	   DO J=N1,M
	      DO I=N1,N
	         XME(5,I,J)=XME(1,I,J)/SIGMOY
	         XME(6,I,J)=XME(4,I,J)/C1-(XME(3,I,J)/C1)**2
	         XME(6,I,J)=DSQRT(XME(6,I,J)/C1)
	      END DO
	   END DO	
	ENDIF
	END SUBROUTINE MESURE3
!--------------------------------------------------------------------------------------
! Nom		: ENTREE 	[SUB]						4
! Fonction	: Lecture des parametres et verification de compatibilite
! Auteurs	: H.Touchette,AM.Tremblay,A.Reid,H.Nelisse,C.Bourbonnais,C.Boily
! Revision	: 15 juillet 1997 (Touchette)
! Notes		: - Le nombre d'option (ZETA) = NOPT
!		  - Le module CONSTANTES fixe toutes les bornes des parametres 
!		  - Voir le fichier d'entree pour une description des parametres
!--------------------------------------------------------------------------------------	
	SUBROUTINE ENTREE(PARA)
	USE FICHIERS,ONLY: FICIN
	USE CONSTANTES
	USE PARA_SIM
	IMPLICIT NONE 
	INTEGER					:: I,FLAG,NR	! Var. locales
	TYPE(PARAMETRE),INTENT(OUT) 		:: PARA
!
	OPEN(UNIT=3,FILE=FICIN,STATUS='OLD',ACCESS='SEQUENTIAL',ERR=666)
	REWIND(UNIT=3)
!
	READ(3,*) PARA%BETA
	READ(3,*) PARA%TRAN
        READ(3,*) PARA%U
        READ(3,*) PARA%MU
        READ(3,*) PARA%REMPLD
        READ(3,*) PARA%TPAR
        READ(3,*) PARA%TPER
        READ(3,*) PARA%TPRM
        READ(3,*) PARA%N
        READ(3,*) PARA%M
        READ(3,*) PARA%MES
        READ(3,*) PARA%NWARM
        READ(3,*) PARA%NMODUL
        READ(3,*) PARA%NBLOC
	READ(3,*) PARA%NIMPRI
        READ(3,*) PARA%NOPT
	DO I=1,PARA%NOPT
	   READ(3,*) PARA%ZETA(I)
	END DO
	READ(3,*) PARA%NR1
	READ(3,*) PARA%NR2
	READ(3,*) PARA%NR3
	READ(3,*) PARA%NR4
	NR=PARA%NR1+PARA%NR2+PARA%NR3+PARA%NR4
	DO I=1,NR
	   READ(3,*) PARA%YNMES(I)
	END DO
!
!	Confirmation de l'entree a l'unite '*'
        WRITE(*,6) PARA%BETA,		'BETA'
        WRITE(*,5) PARA%TRAN,		'TRAN'
        WRITE(*,6) PARA%U,		'U'
        WRITE(*,6) PARA%MU,		'MU'
        WRITE(*,6) PARA%REMPLD,		'REMPLD'
        WRITE(*,6) PARA%TPAR,		'TPAR'
        WRITE(*,6) PARA%TPER,		'TPER'
        WRITE(*,6) PARA%TPRM,		'TPRM'
        WRITE(*,5) PARA%N,		'N'
        WRITE(*,5) PARA%M,		'M'
        WRITE(*,5) PARA%MES,		'MES'
        WRITE(*,5) PARA%NWARM,		'NWARM'
        WRITE(*,5) PARA%NMODUL,		'NMODUL'
        WRITE(*,5) PARA%NBLOC,		'NBLOC'
        WRITE(*,5) PARA%NIMPRI,		'NIMPRI'
        WRITE(*,5) PARA%NOPT,		'NOPT'
        DO I=1,PARA%NOPT
           WRITE(*,*) PARA%ZETA(I)
        END DO
        WRITE(*,5) PARA%NR1,		'NR1'
        WRITE(*,5) PARA%NR2,		'NR2'
        WRITE(*,5) PARA%NR3,		'NR3'
        WRITE(*,5) PARA%NR4,		'NR4'
        DO I=1,NR
           WRITE(*,*) PARA%YNMES(I)
	END DO
!
!	Verifications
	FLAG=0
166	IF(MOD(PARA%TRAN,PARA%NMODUL).NE.0)   				GOTO 667
168	IF(MOD(PARA%MES,PARA%NBLOC).NE.0)				GOTO 668
169	IF(PARA%NBLOC.GT.PARA%MES) 					GOTO 669
170	IF(PARA%N.GT.ND1) 						GOTO 670
171	IF(PARA%M.GT.ND2) 						GOTO 671
172	IF(PARA%TRAN.GT.NL) 						GOTO 672
173	IF(PARA%MES/PARA%NBLOC.GT.NCOR) 				GOTO 673
174	IF(PARA%NR1.GT.NSC)						GOTO 674
175	IF(PARA%NR2.GT.NTE)						GOTO 675
176	IF(PARA%NR3.GT.NTI)						GOTO 676
177	IF(PARA%NR4.GT.NTG)						GOTO 677
178	IF((PARA%YNMES(9).EQ.'Y'.OR.PARA%YNMES(9).EQ.'y').AND.&
	  &(PARA%YNMES(8).NE.'Y'.AND.PARA%YNMES(8).NE.'y'))		GOTO 678
179	IF((PARA%YNMES(10).EQ.'Y'.OR.PARA%YNMES(10).EQ.'y').AND.&
	  &(PARA%YNMES(9).NE.'Y'.AND.PARA%YNMES(9).NE.'y'))		GOTO 679 
180	IF((PARA%ZETA(5).EQ.'Y'.OR.PARA%ZETA(5).EQ.'y').AND.&
	  &(PARA%YNMES(8).NE.'Y'.AND.PARA%YNMES(8).NE.'y'))		GOTO 680
181	IF(FLAG.EQ.1) STOP
!
	CLOSE(UNIT=3)
!
5	FORMAT(I10,A10)
6	FORMAT(F10.5,A10)
	RETURN
!	Messages d'erreurs
666	WRITE(*,*) '--- Erreur : Fichier inexistant (ENTREE) ---'; STOP
667	WRITE(*,*) '--- Erreur : TRAN pas multiple de NMODUL ---'; 	FLAG=1;GOTO 168
668	WRITE(*,*) '--- Erreur : MES pas multiple NBLOC ---'; 	  	FLAG=1;GOTO 169
669	WRITE(*,*) '--- Erreur : NBLOC > MES ---'; 		   	FLAG=1;GOTO 170
670	WRITE(*,*) '--- Erreur : N > ND1 ---'; 			   	FLAG=1;GOTO 171
671	WRITE(*,*) '--- Erreur : M > ND2 ---'; 			   	FLAG=1;GOTO 172
672	WRITE(*,*) '--- Erreur : TRAN > NL ---'; 			FLAG=1;GOTO 173
673	WRITE(*,*) '--- Erreur : Nb de blocs > NCOR ---'; 		FLAG=1;GOTO 174 
674	WRITE(*,*) '--- Erreur : NR1 > NSC ---'; 			FLAG=1;GOTO 175
675	WRITE(*,*) '--- Erreur : NR2 > NTE ---'; 			FLAG=1;GOTO 176
676	WRITE(*,*) '--- Erreur : NR3 > NTI ---'; 			FLAG=1;GOTO 177
677	WRITE(*,*) '--- Erreur : NR4 > NTG ---'; 			FLAG=1;GOTO 178
678	WRITE(*,*) '--- Erreur : MATSUBARA sans fct de Green ---'; 	FLAG=1;GOTO 179
679	WRITE(*,*) '--- Erreur : SELF sans MATSUBARA ---'; 		FLAG=1;GOTO 180
680	WRITE(*,*) '--- Erreur : COVARIANCE sans fcts de GREEN ---';	FLAG=1;GOTO 181
	END SUBROUTINE
!--------------------------------------------------------------------------------------
! Nom		: SORTIE1 	[SUB]						5-(1)
! Fonction	: Impressions des resultats
! Auteurs	: H.Touchette
! Revision	: 15 juillet 1997 (Touchette)
! Notes		: - UNIT=2 
!		    
!--------------------------------------------------------------------------------------
	SUBROUTINE SORTIE1(PARA,MESC,COMP,SIGMOY,FLUCT,XMUPROP,ENERG,OX,VX)	
	USE CONSTANTES,ONLY: NXC
	USE PARA_SIM
	USE IDENT
	IMPLICIT NONE
	INTEGER						:: NM
	REAL(8),INTENT(IN)				:: SIGMOY
	REAL(8),INTENT(IN)				:: FLUCT,XMUPROP,ENERG
	INTEGER,INTENT(IN)				:: MESC	
	REAL(8),DIMENSION (2)				:: COMP	
	REAL(8),DIMENSION (0:NXC),INTENT(IN)		:: OX,VX
	TYPE(PARAMETRE),INTENT(IN) 			:: PARA
!
	NM=PARA%N*PARA%M
!
!	Sortie des parametres et resultats au fichier (UNIT=2)
	WRITE(2,*) '------------------------------------------------------------'
	WRITE(2,25)  DTE,		'Date mm/dd/yy'
	WRITE(2,23) IDNUM,		'No. simulation'
	WRITE(2,*)  NANUM,		'Prefixe fichier'
	WRITE(2,23) MESC,		'Nb mesures effectuees'
	WRITE(2,*) ' '
	WRITE(2,*) 'Parametres de simulation'
	WRITE(2,*) ' '
	WRITE(2,17) PARA%BETA,		'BETA'
	WRITE(2,15) PARA%TRAN,		'TRAN'
	WRITE(2,17) PARA%U,		'U'
	WRITE(2,17) PARA%MU,		'MU'
	WRITE(2,17) PARA%REMPLD,	'Remplissage desire'
	WRITE(2,17) PARA%TPAR,		'TPAR'
	WRITE(2,17) PARA%TPER,		'TPER'
	WRITE(2,17) PARA%TPRM,		'TPRM'
	WRITE(2,15) PARA%N,		'Nombre de sites par chaine N'
	WRITE(2,15) PARA%M,		'Nombre de chaines M'
	WRITE(2,15) PARA%MES,		'Nombre total de mesures'
	WRITE(2,15) PARA%NWARM,		'Nombre de rechauffement'
	WRITE(2,15) PARA%NMODUL,	'NMODUL : Nb de tranche de stabilisation'
	WRITE(2,15) PARA%NBLOC,		'Nombre de mesures dans un bloc'
	WRITE(2,15) PARA%NIMPRI,	'Borne impression'
	WRITE(2,15) PARA%NOPT,		'Nombre option de calcul'  
	WRITE(2,13) PARA%ZETA(1),	'Fct autocorrelation'
	WRITE(2,13) PARA%ZETA(2),	'Lecture configuration HS initiale'
	WRITE(2,13) PARA%ZETA(3),	'Sauvegarde des configuration HS'
	WRITE(2,13) PARA%ZETA(4),	'Mouvements globaux'
	WRITE(2,13) PARA%ZETA(5),	'Matrice de covariance'
	WRITE(2,15) PARA%NR1,		'Nombre de routines scalaire'
	WRITE(2,15) PARA%NR2,		'Nombre de routines temps egal'
	WRITE(2,15) PARA%NR3,		'Nombre de routines temps inegal'
	WRITE(2,15) PARA%NR4,		'Nombre de routines fct de Green'
	WRITE(2,13) PARA%YNMES(1),	'DSITE'
	WRITE(2,13) PARA%YNMES(2),	'MAGST'
	WRITE(2,13) PARA%YNMES(3),	'DENSITE'
	WRITE(2,13) PARA%YNMES(4),	'RHOE'
	WRITE(2,13) PARA%YNMES(5),	'PSCOR'
	WRITE(2,13) PARA%YNMES(6),	'Integrale sur tau de SUSCEPI'
	WRITE(2,13) PARA%YNMES(7),	'SUSCEPI'
	WRITE(2,13) PARA%YNMES(8),	'Fct de Green (qx,qy)'
	WRITE(2,13) PARA%YNMES(9),	'MATSUBARA G(iwn,Qx,Qy)'
	WRITE(2,13) PARA%YNMES(10),	'SELF'
	WRITE(2,*) ' '
	WRITE(2,*) 'Parametres de la bande'
	WRITE(2,*) ' '
	WRITE(2,20) OX(0),VX(0),	'Remplissage mesure'
	WRITE(2,21) XMUPROP,		'Mu propose'
	WRITE(2,21) COMP(1),		'Proportion de spin-flip accepte'
	WRITE(2,21) COMP(2),		'Proportion de global move accepte'
	WRITE(2,21) SIGMOY/DBLE(MESC),	'Signe moyen du produit des det.'
	WRITE(2,*) ' '
	WRITE(2,20) OX(1)/NM,VX(1)/NM,	'Energie cinetique'
	WRITE(2,20) OX(2)/NM,VX(2)/NM,	'Energie totale'
	WRITE(2,21) ENERG,		'Energie des electrons libres'
	WRITE(2,21) FLUCT,		'Fluctuation du nombre de particules'
!
!	Format d'impression
13	FORMAT(A2,2x,A)
15	FORMAT(I10,2x,A)
17	FORMAT(F10.7,2x,A)
20	FORMAT(F10.7,1x,F10.7,2x,A)
21	FORMAT(F10.7,2x,A)
23	FORMAT(I9,12X,A)
25	FORMAT(A9,12X,A)
!
	END SUBROUTINE SORTIE1
!--------------------------------------------------------------------------------------
! Nom		: SORTIE2 	[SUB]						5-(2)
! Fonction	: Impressions des facteurs de structures a temps egal et inegal
! Auteurs	: H.Touchette
! Revision	: 20 juillet 1997 (Touchette)
! Notes		: - Utilise WRITEOUT par l'intermediare de l'interface INTER
!                 - UNIT=2
!--------------------------------------------------------------------------------------
	SUBROUTINE SORTIE2(PARA,X,VX,XI,VXI)
	USE CONSTANTES,ONLY: NL,ND1,ND2,NTE,NTI
	USE PARA_SIM
	USE INTER
	IMPLICIT NONE
	REAL(8),DIMENSION (NTE,ND1,ND2)			:: X,VX
	REAL(8),DIMENSION (NTI,0:NL,ND1,ND2)		:: XI,VXI
	TYPE(PARAMETRE),INTENT(IN)			:: PARA 
!
	WRITE(2,*) ' ' 
	WRITE(2,*) 'Quantites a temps egal'
	WRITE(2,*) ' '
	IF(PARA%YNMES(2).EQ.'Y'.OR.PARA%YNMES(2).EQ.'y') THEN
	   WRITE(2,*) 'Susceptibilite magnetique (MAGST)'
	   WRITE(2,*) 'qx    qy      x         err(x)'
	   WRITE(2,*) ' '
	   CALL WRITEOUT(PARA,2,E=X(1,:,:),VE=VX(1,:,:))
	ENDIF
	IF(PARA%YNMES(3).EQ.'Y'.OR.PARA%YNMES(3).EQ.'y') THEN
	   WRITE(2,*) 'Densite impulsion (DENSITE)'
	   WRITE(2,*) 'qx    qy      k         err(k)'
	   WRITE(2,*) ' '
	   CALL WRITEOUT(PARA,2,E=X(2,:,:),VE=VX(2,:,:))
	ENDIF
	IF(PARA%YNMES(4).EQ.'Y'.OR.PARA%YNMES(4).EQ.'y') THEN
	   WRITE(2,*) 'Densite de charge (RHOE)'
	   WRITE(2,*) 'qx    qy      x         err(x)'
	   WRITE(2,*) ' '
	   CALL WRITEOUT(PARA,2,E=X(3,:,:),VE=VX(3,:,:))
	ENDIF
	IF(PARA%YNMES(5).EQ.'Y'.OR.PARA%YNMES(5).EQ.'y') THEN
	   WRITE(2,*) 'Susceptibilite de paires (PSCOR)'
	   WRITE(2,*) 'qx    qy      x         err(x)'
	   WRITE(2,*) ' '
	   CALL WRITEOUT(PARA,2,E=X(4,:,:),VE=VX(4,:,:))
	ENDIF
	IF(PARA%YNMES(6).EQ.'Y'.OR.PARA%YNMES(6).EQ.'y') THEN
	   WRITE(2,*) 'Integrale sur tau de la susceptibilite de paire dynamique'
	   WRITE(2,*) 'qx    qy      x         err(x)'
	   WRITE(2,*) ' '
	   CALL WRITEOUT(PARA,2,E=X(5,:,:),VE=VX(5,:,:))
	ENDIF
	WRITE(2,*) 'Quantites a temps inegal'
	WRITE(2,*) ' '
	IF(PARA%YNMES(7).EQ.'Y'.OR.PARA%YNMES(7).EQ.'y') THEN
	   WRITE(2,*) 'Susceptibilite de paires dynamique (SUSCEPI)'
	   WRITE(2,*) ' qx  qy  tau     X            err'
	   WRITE(2,*) ' '
	   CALL WRITEOUT(PARA,2,F=XI(1,:,:,:),VF=VXI(1,:,:,:))
	ENDIF
!
	END SUBROUTINE SORTIE2
!--------------------------------------------------------------------------------------
! Nom		: SORTIE3 	[SUB]						5-(3)
! Fonction	: Impressions des fonctions de Green
! Auteurs	: H.Touchette
! Revision	: 20 juillet 1997 (Touchette)
! Notes		: - Utilise WRITEOUT par l'intermediare de l'interface INTER
!		  - UNIT=11
!--------------------------------------------------------------------------------------
	SUBROUTINE SORTIE3(PARA,MESC,GU,GD,VGU,VGD)
	USE CONSTANTES,ONLY: NL,ND1,ND2,NTG
	USE PARA_SIM
	USE INTER
	USE IDENT,ONLY: IDNUM
	IMPLICIT NONE
	INTEGER,INTENT(IN)				:: MESC
	REAL(8),DIMENSION (NTG,0:NL,ND1,ND2),INTENT(IN)	:: GU,GD 
	REAL(8),DIMENSION (NTG,0:NL,ND1,ND2),INTENT(IN)	:: VGU,VGD
	TYPE(PARAMETRE),INTENT(IN)			:: PARA 
!
	WRITE(11,*) '------------------------------------------------------------'
	WRITE(11,97) IDNUM,		'No. simulation'
	WRITE(11,97) MESC,		'Nb mesures effectuees'
	WRITE(11,*) ' '
	IF(PARA%YNMES(8).EQ.'Y'.OR.PARA%YNMES(8).EQ.'y') THEN
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Fonctions de Green, espace reciproque'
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Spin-up'
	   WRITE(11,*) ' qx  qy  tau     G            err'
	   CALL WRITEOUT(PARA,11,F=GU(1,:,:,:),VF=VGU(1,:,:,:))
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Spin-down'
	   WRITE(11,*) ' qx  qy  tau     G            err'
	   CALL WRITEOUT(PARA,11,F=GD(1,:,:,:),VF=VGD(1,:,:,:))
	ENDIF
	IF(PARA%YNMES(9).EQ.'Y'.OR.PARA%YNMES(9).EQ.'y') THEN
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Fonctions de Green de Matsubara, espace reciproque'
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Partie reelle'
	   WRITE(11,*) ' qx  qy  tau     G            err'
	   CALL WRITEOUT(PARA,11,F=GU(2,:,:,:),VF=VGU(2,:,:,:))
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Partie imaginaire'
	   WRITE(11,*) ' qx  qy  tau     G            err'
	   CALL WRITEOUT(PARA,11,F=GD(2,:,:,:),VF=VGD(2,:,:,:))
	ENDIF
	IF(PARA%YNMES(10).EQ.'Y'.OR.PARA%YNMES(10).EQ.'y') THEN
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Self-energie, espace reciproque'
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Partie reelle'
	   WRITE(11,*) ' qx  qy  tau     S            err'
	   CALL WRITEOUT(PARA,11,F=GU(3,:,:,:),VF=VGU(3,:,:,:))
	   WRITE(11,*) ' '
	   WRITE(11,*) 'Partie imaginaire'
	   WRITE(11,*) ' qx  qy  tau     S            err'
	   CALL WRITEOUT(PARA,11,F=GD(3,:,:,:),VF=VGD(3,:,:,:))
	ENDIF
	WRITE(11,*) ' '
97	FORMAT(I9,12X,A)
98 	FORMAT(2(I4),2(I6),3x,F10.7)
!
	END SUBROUTINE SORTIE3
!--------------------------------------------------------------------------------------
! Nom		: SORTIEC 	[SUB]						5-(4)
! Fonction	: Impressions de la fonction d'autocorrelation et de la matrice 
!		  de covariance
! Auteurs	: H.Touchette
! Revision	: 6 aout 1997 (Touchette)
! Notes		: - UNIT 15
!		  - Les valeurs de la matrice de covariance pour la derniere tranche
!		    de temps imaginaire ne sont pas imprimees du fait qu'elles 
!		    derivent exactement des valeurs a tau = 0. 
!--------------------------------------------------------------------------------------
	SUBROUTINE SORTIEC(PARA,MESC,C1,AUTOCOR,COVM)
	USE CONSTANTES,ONLY: NL,ND1,ND2,NCOR 
	USE FICHIERS,ONLY: FICCVOUT
	USE IDENT,ONLY: IDNUM
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER							:: I,J,L1,L2
	INTEGER							:: NC1
	REAL(8),INTENT(IN)					:: C1
	INTEGER,INTENT(IN)					:: MESC
	REAL(8),DIMENSION (NCOR),INTENT(IN)			:: AUTOCOR
	REAL(8),DIMENSION (0:NL,0:NL,ND1,ND2),INTENT(IN)	:: COVM
	TYPE(PARAMETRE),INTENT(IN)				:: PARA 
!
	NC1=INT(C1)
	OPEN(UNIT=15,FILE=FICCVOUT)
	REWIND(UNIT=15)
	WRITE(15,77) IDNUM,		'No. simulation'
	WRITE(15,77) MESC,		'Nb mesures effectuees'
	WRITE(15,77) NC1,		'Nb blocs'
	WRITE(15,*) ' '
	IF(PARA%ZETA(5).EQ.'Y'.OR.PARA%ZETA(5).EQ.'y') THEN
	   WRITE(15,*) ' '
	   WRITE(15,*) 'Matrice de covariance spin-up'
	   WRITE(15,*) ' '
	   WRITE(15,*) ' qx   qy    tau1   tau2         C'
	   DO I=1,PARA%N
	      DO J=1,PARA%M
	         DO L1=0,PARA%TRAN-1
	            DO L2=0,PARA%TRAN-1
	               WRITE(15,78) I,J,L1,L2,COVM(L1,L2,I,J)
	            END DO
	         END DO
	      END DO
	   END DO
	ENDIF
	IF(PARA%ZETA(1).EQ.'Y'.OR.PARA%ZETA(1).EQ.'y') THEN
	   WRITE(15,*) ' '
	   WRITE(15,*) 'Fonction autocorrelation'
	   WRITE(15,*) ' '
	   WRITE(15,*) '  i    n         C(n)'
	   DO I=1,NC1
	      WRITE(15,79) I,I-1,AUTOCOR(I)
	   END DO	               
	ENDIF
	CLOSE(UNIT=15)
!
77	FORMAT(I9,12X,A)
78	FORMAT(2(I4,1X),2(I6,1X),E16.4)
79	FORMAT(2(I4,1x),E16.4)
	END SUBROUTINE SORTIEC
!--------------------------------------------------------------------------------------
! Nom		: SORTIE4 	[SUB]						5-(5)
! Fonction	: Temps de calcul et archivage des simulations pour les pages web
! Auteurs	: H.Touchette
! Revision	: 22 juillet 1997 (Touchette)
! Notes		: UNIT=2  : Temps de calcul
!		  UNIT=13 : Archivage
!--------------------------------------------------------------------------------------
	SUBROUTINE SORTIE4(PARA,XMU,REMP)
	USE FICHIERS,ONLY: FICWEB,FICPAOUT,FICHSOUT,FICGIOUT,REPWEB
	USE PARA_SIM
	USE IDENT
	IMPLICIT NONE
	REAL(8)				:: TF,TD
	REAL(8),INTENT(IN)		:: XMU,REMP
	INTEGER				:: IO,IH,IG,IR		! Variables locales
	INTEGER				:: MCLOCK		! Horloge CPU
	TYPE(PARAMETRE),INTENT(IN)	:: PARA
!
	TF=MCLOCK()						! Heure d'arrivee
	TD=(TF-TI)/6000						! Diff en minutes
	WRITE(2,*) ' '
	WRITE(2,*) TD,'  = Temps de calcul (minutes CPU)'
!
	IO=INDEX(FICPAOUT,' ')
	IH=INDEX(FICHSOUT,' ')
	IG=INDEX(FICGIOUT,' ')
	IR=INDEX(REPWEB,' ')
	OPEN(UNIT=17,FILE=FICWEB,POSITION="APPEND")
	WRITE(17,*) '<PRE>'
	WRITE(17,102) DTE,IDNUM,PARA%N,PARA%M,PARA%U,PARA%BETA,PARA%MU,XMU,REMP,&
	              &PARA%MES,PARA%NWARM,PARA%NBLOC,TD,REPWEB
	WRITE(17,103) '<A HREF="',REPWEB(1:IR-1),'/',FICPAOUT,'">',FICPAOUT,'</A>'
	WRITE(17,103) '<A HREF="',REPWEB(1:IR-1),'/',FICHSOUT,'">',FICHSOUT,'</A>'
	WRITE(17,103) '<A HREF="',REPWEB(1:IR-1),'/',FICGIOUT,'">',FICGIOUT,'</A>'
	WRITE(17,*) '</PRE>'
	CLOSE(UNIT=17)
102	FORMAT(A8,1X,I8,2(1X,I2),5(1X,F6.2),3(1X,I8),1X,E16.2,6X,A)
103	FORMAT(106X,A9,A<IR-1>,A1,A<IO-1>,A2,A<IO>,A4,&
	      &  1X,A9,A<IR-1>,A1,A<IH-1>,A2,A<IH>,A4,&
	      &  1X,A9,A<IR-1>,A1,A<IG-1>,A2,A<IG>,A4)
	END SUBROUTINE SORTIE4
!--------------------------------------------------------------------------------------
! Nom		: WRITEOUT		[SUB]					6
! Fonction	: Ecriture des resultats
! Auteurs	: H.Touchette
! Revision	: 20 juin 1997 (Touchette)
! Notes		: - Cette routine a ete construite de facon a illustrer l'utilisation
!		    d'arguments optionnels a l'aide d'une interface explicite
!		  - Utilitaires de SORTIE2
!		  - E : Resultats a temps egal UNIT=2 
!		  - F : Resultats a temps inegal UNIT 2 ou 11 (NU)
!--------------------------------------------------------------------------------------
	SUBROUTINE WRITEOUT(PARA,NU,E,VE,F,VF)
	USE CONSTANTES,ONLY: ND1,ND2,NL
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER							:: I,J,L
	INTEGER,INTENT(IN)					:: NU
	REAL(8),DIMENSION (ND1,ND2),INTENT(IN),OPTIONAL		:: E,VE
	REAL(8),DIMENSION (0:NL,ND1,ND2),INTENT(IN),OPTIONAL	:: F,VF
	TYPE(PARAMETRE),INTENT(IN)				:: PARA
!
	IF(PRESENT(E)) THEN
	   DO I=1,PARA%N
	      DO J=1,PARA%M
	         WRITE(NU,324) I,J,E(I,J),VE(I,J)
	      END DO
	   END DO
	ELSE IF(PRESENT(F)) THEN
	   DO I=1,PARA%N
	      DO J=1,PARA%M
	         DO L=0,PARA%TRAN
	            WRITE(NU,325) I,J,L,F(L,I,J),VF(L,I,J)
	         END DO
	      END DO
	   END DO
	ELSE
	   WRITE(*,*) '--- Erreur : WRITEOUT ---'
	ENDIF
324	FORMAT(2(1X,I3),2(2X,F10.7))
325	FORMAT(3(1X,I3),2(1X,F10.7))
	END SUBROUTINE WRITEOUT
!--------------------------------------------------------------------------------------
! Nom		: GRASCHP	[SUB]						7
! Fonction	: Decomposition UDV par la methode de Gram-Schmidt modifiee + pivot
! Auteurs	: 
! Revision	: 
! References	: S.Moukouri
! Notes		: 
! Dependances	: ORTHOGP,DONORM (internes)
!--------------------------------------------------------------------------------------
	SUBROUTINE GRASCHP(NP,NM,AMAT,DVEC,RMAT)
	IMPLICIT NONE
	REAL(8)					:: TEMP		! Variable locale
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER					:: I,J		! Variables locales
	REAL(8),DIMENSION (NP),INTENT(OUT)	:: DVEC
	REAL(8),DIMENSION (NP,NP),INTENT(OUT)	:: RMAT
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: AMAT
!
 	CALL ORTHOGP(NP,NM,AMAT,RMAT)
 	DO I=1,NM
 	   DVEC(I)=RMAT(I,I)
 	   IF(DVEC(I).EQ.0.0) THEN
 	      RMAT(I,I)=1.D0
 	   ELSE IF(DVEC(I).NE.1.0) THEN
 	      TEMP=1.D0/DVEC(I)
 	      DO J=1,NM
 	         RMAT(I,J)=RMAT(I,J)*TEMP
 	      END DO
 	   END IF
 	END DO
 	RETURN
 	END
!
!	Routines internes
!
 	SUBROUTINE ORTHOGP(NP,NM,AMAT,RMAT)
 	IMPLICIT NONE
 	REAL(8) 				:: AMAXSIZE,COLSIZE	! Loc.
 	INTEGER,INTENT(IN)			:: NP,NM
 	INTEGER					:: I,J,K,IB	! Variables locales
 	REAL(8),DIMENSION (NP)			:: IDONE	! Variable locale	
 	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: AMAT
	REAL(8),DIMENSION (NP,NP),INTENT(OUT)	:: RMAT
  	DO I=1,NM
 	   DO J=1,NM
 	      RMAT(I,J)=0.D0
 	   END DO
 	   RMAT(I,I)=1.D0
 	END DO
  	DO I=1,NM
 	   IDONE(I)=0
 	END DO
 	DO I=1,NM
 	   AMAXSIZE=-10.0
 	   IB=-1
 	   DO J=1,NM
 	      IF(IDONE(J).EQ.0) THEN
 	         COLSIZE=0.0
 	         DO K=1,NM
 	            COLSIZE=COLSIZE+AMAT(K,J)**2
 	         END DO
 	         IF(COLSIZE.GT.AMAXSIZE) THEN
 	            AMAXSIZE=COLSIZE
 	            IB=J
 	         END IF
 	      END IF
 	   END DO
 	   IDONE(IB)=1
 	   CALL DONORM(NP,NM,AMAT(1,IB),RMAT(IB,IB))
 	   DO J=1,NM
 	      IF(IDONE(J).EQ.0) THEN
 	         DO K=1,NM
 	            RMAT(IB,J)=RMAT(IB,J)+AMAT(K,IB)*AMAT(K,J)
 	         END DO
 	         DO K=1,NM
 	            AMAT(K,J)=AMAT(K,J)-RMAT(IB,J)*AMAT(K,IB)
 	         END DO
 	      END IF
 	   END DO
 	END DO
 	END SUBROUTINE ORTHOGP
!
 	SUBROUTINE DONORM(NP,NM,VEC,VNORM)
 	IMPLICIT NONE
 	REAL(8) 				:: VNORM,TEMP	! Variables locales
 	INTEGER					:: I		! Variable locale
 	INTEGER,INTENT(IN)			:: NP,NM
 	REAL(8),DIMENSION (NP),INTENT(INOUT)	:: VEC
  	TEMP=0.D0
 	DO I=1,NM
 	   TEMP=TEMP+VEC(I)*VEC(I)
 	END DO
 	VNORM=DSQRT(TEMP)
 	IF(VNORM.NE.0.0) THEN
 	   TEMP=1.D0/VNORM
 	   DO I=1,NM
 	      VEC(I)=VEC(I)*TEMP
 	   END DO
 	END IF
 	END SUBROUTINE DONORM
!--------------------------------------------------------------------------------------
! Nom		: MULD		[SUB]						8
! Fonction	: Multiplication par une matrice diagonale
! Auteurs	: AM.Tremblay
! Revision	: 16 aout 1989 (Tremblay)
! Notes		: L=0 : multiplication par la gauche A = D * A (D:diag)
!		  L=2 : multiplication par la droite A = A * D
!--------------------------------------------------------------------------------------
	SUBROUTINE MULD(NP,NM,L,A,D)
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM,L
	INTEGER					:: I,J
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: A
	REAL(8),DIMENSION (NP),INTENT(IN)	:: D
	IF(L.EQ.0) THEN						! L = 0						
	   DO I=1,NM
	      DO J=1,NM
	         A(I,J)=D(I)*A(I,J)
	      END DO
	   END DO
	ELSE IF(L.EQ.2) THEN					! L = 2
	   DO J=1,NM
	      DO I=1,NM
	         A(I,J)=A(I,J)*D(J)
	      END DO
	   END DO
	ELSE							! Erreur
	   WRITE(*,*) '--- Erreur : MULD ---'
	ENDIF
	END SUBROUTINE MULD
!--------------------------------------------------------------------------------------
! Nom		: MULTD		[SUB]						9
! Fonction	: Multiplication par une matrice triangulaire droite
! Auteurs	: AM.Tremblay
! Revision	: 16 aout 1989 (Tremblay)
! Notes		: L=0 : multiplication par la gauche R = A(transpose) * T
!		  L=2 : multiplication par la droite R = T * A(transpose)
!			T = triangulaire droite NM*NM
!			A = Matrice source (par laquelle on fait la multiplication)
!			R = Resultat de la multiplication
!--------------------------------------------------------------------------------------
	SUBROUTINE MULTD(NP,NM,L,A,T,R)
	IMPLICIT NONE
	REAL(8)					:: S		! Variable locale
	INTEGER,INTENT(IN)			:: NP,NM,L
	INTEGER					:: I,J,K	! Variables locales
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: A
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: T
	REAL(8),DIMENSION (NP,NP),INTENT(OUT)	:: R
	IF(L.EQ.2) THEN						! L = 2
	   DO I=1,NM
	      DO J=1,NM
	         S=0.D0
	         DO K=I,NM
	            S=S+T(I,K)*A(J,K)
	         END DO
	         R(I,J)=S
	      END DO
	   END DO
	ELSE IF(L.EQ.0) THEN					! L = 0
	   DO I=1,NM
	      DO J=1,NM
	         S=0.D0
	         DO K=1,J
	            S=S+A(K,I)*T(K,J)
	         END DO
	         R(I,J)=S
	      END DO
	   END DO
	ELSE							! Erreur
	   WRITE(*,*) '--- Erreur : MULTD ---'
	ENDIF
	END SUBROUTINE MULTD
!--------------------------------------------------------------------------------------
! Nom		: MULTD2	[SUB]						10
! Fonction	: Multiplication de deux matrices triangulaires droites
! Auteurs	: AM.tremblay
! Revision	: 16 aout 1989 (Tremblay)
! Notes		: T2 = T1 * T2 (T1,T2: triangulaires droites NM*NM)
!--------------------------------------------------------------------------------------
	SUBROUTINE MULTD2(NP,NM,T1,T2)
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER					:: I,J,K,L	! Variables locales
	REAL(8),DIMENSION (NP)			:: R		! Variable locale
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: T1
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: T2
	DO I=1,NM
	   DO J=1,I
	      R(J)=0.D0
	      DO K=J,I
	         R(J)=R(J)+T1(J,K)*T2(K,I)
	      END DO
	   END DO
	   DO L=1,I
	      T2(L,I)=R(L)
	   END DO
	END DO
	END SUBROUTINE MULTD2
!--------------------------------------------------------------------------------------
! Nom		: MAC1		[SUB]						11-(1)
! Fonction	: A * B => B
! Auteurs	:
! Revision	:
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC1(NP,NM,A,B)
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER					:: I,J,K,L	! Variables locales
	REAL(8),DIMENSION (NP)			:: C		! Variable locale
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: A
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: B
	DO I=1,NM
	   DO J=1,NM
	      C(J)=0.D0
	      DO K=1,NM
	         C(J)=C(J)+A(J,K)*B(K,I)
	      END DO
	   END DO
	   DO L=1,NM
	      B(L,I)=C(L)
	   END DO
	END DO
	END SUBROUTINE MAC1
!--------------------------------------------------------------------------------------
! Nom		: MAC2		[SUB]						11-(2)
! Fonction	: A * B => A
! Auteurs	:
! Revision	:
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC2(NP,NM,A,B)
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER					:: I,J,K,L	! Variables locales
	REAL(8),DIMENSION (NP)			:: C		! Variable locale
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: A
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: B
	DO I=1,NM
	   DO J=1,NM
	      C(J)=0.D0
	      DO K=1,NM
	         C(J)=C(J)+A(I,K)*B(K,J)
	      END DO
	   END DO
	   DO L=1,NM
	      A(I,L)=C(L)
	   END DO
	END DO
	END SUBROUTINE MAC2
!--------------------------------------------------------------------------------------
! Nom		: MAC8		[SUB]						11-(8)
! Fonction	: CTE* A => B
! Auteurs	:
! Revision	:
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC8(NP,NM,A,B,CTE)
	IMPLICIT NONE
	REAL(8),INTENT(IN)			:: CTE
	INTEGER,INTENT(IN)			:: NP,NM	
	INTEGER					:: I,J		! Variables locales
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: A
	REAL(8),DIMENSION (NP,NP),INTENT(OUT)	:: B
	DO J=1,NM
	   DO I=1,NM
	      B(I,J)=CTE*A(I,J)
	   END DO
	END DO
	END SUBROUTINE MAC8
!--------------------------------------------------------------------------------------
! Nom		: MAC9		[SUB]						11-(9)
! Fonction	: A + I => A
! Auteurs	:
! Revision	:
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC9(NP,NM,A)
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER					:: I		! Variable locale
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: A
	DO I=1,NM
	   A(I,I)=A(I,I)+1.D0
	END DO
	END SUBROUTINE MAC9
!--------------------------------------------------------------------------------------
! Nom		: MAC10		[SUB]						11-(10)
! Fonction	: A => B
! Auteurs	:
! Revision	:
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC10(NP,NM,A,B)
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER					:: I,J		! Variables locales
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: A
	REAL(8),DIMENSION (NP,NP),INTENT(OUT)	:: B
	DO J=1,NM
	   DO I=1,NM
	      B(I,J)=A(I,J)
	   END DO
	END DO
	END SUBROUTINE MAC10
!--------------------------------------------------------------------------------------
! Nom		: MAC11		[SUB]						11-(11)
! Fonction	: TRANSPOSE(A) => A
! Auteurs	:
! Revision	: 30 juillet 97 (Touchette)
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC11(NP,NM,A)
	IMPLICIT NONE
	REAL(8)					:: C 		! Var. locale
	INTEGER					:: I,J		! Var. locales
	INTEGER,INTENT(IN)			:: NP,NM
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: A			
	DO I=2,NM
	   DO J=1,I-1
	      C=A(I,J)
	      A(I,J)=A(J,I)
	      A(J,I)=C
	   END DO
	END DO
	END SUBROUTINE MAC11
!--------------------------------------------------------------------------------------
! Nom		: INVTD		[SUB]						12
! Fonction	: Inversion d'une matrice triangulaire droite
! Auteurs	: AM.Tremblay
! Revision	: 16 aout 1989 (Tremblay)
! Notes		: - A: triangulaire droite NM*NM
!		  - Ne sert pas dans le programme
!--------------------------------------------------------------------------------------
	SUBROUTINE INVTD(NP,NM,A)
	IMPLICIT NONE
	REAL(8)					:: S		! Variable locale
	INTEGER					:: I,J,K	! Variables locales
	INTEGER,INTENT(IN)			:: NP,NM
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: A			
        REAL(8),DIMENSION (NP,NP)		:: R		! Variable locale 
        DO J=1,NM	
           DO I=1,NM
              R(I,J)=0.D0
           END DO
           R(I,I)=1.D0
        END DO
	DO K=NM,2,-1
	   DO I=K-1,1,-1
	      S=0.D0
	      DO J=I+1,K
	         S=S-A(I,J)*R(J,K)
	      END DO
	      R(I,K)=S
	   END DO
	END DO
	CALL MAC10(NP,NM,R,A)
	END SUBROUTINE INVTD
!--------------------------------------------------------------------------------------
! Nom		: UPDTG		[SUB]						13
! Fonction	: Mise a jour de la fonction de Green si un changement de spin HS
!		  a ete accepte.
! Auteurs	: AM.Tremblay
! Revision	:
! Reference	: Blankenbecler et al. (1981)
! Notes		:
!--------------------------------------------------------------------------------------	
	SUBROUTINE UPDTG(NP,NM,NS,G,VAR)
	IMPLICIT NONE
	REAL(8)					:: FACT		! Variables locales
	REAL(8),INTENT(IN)			:: VAR
	INTEGER,INTENT(IN)			:: NP,NM,NS
	INTEGER					:: I,J		! Variables locales
	REAL(8),DIMENSION (NP)			:: VV,VH	! Variables locales
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: G
!
	FACT=1.D0+(1.D0-G(NS,NS))*VAR
	FACT=VAR/FACT
	DO I=1,NM
	   VV(I)=G(I,NS)
	   VH(I)=G(NS,I)
	END DO
	DO I=1,NM
	   DO J=1,NM
	      G(I,J)=G(I,J)+VV(I)*VH(J)*FACT
	   END DO
	END DO
	DO J=1,NM
	   G(NS,J)=G(NS,J)-VH(J)*FACT
	END DO
	END SUBROUTINE UPDTG
!--------------------------------------------------------------------------------------
! Nom		: INITH3	[SUB]						14
! Fonction	: Initialisation de la matrice de hopping (forme dense diagonalisee)
! Auteurs	: Liang Chen,H.Touchette
! Revision	: 21 novembre 1996 (Touchette)
! Notes		: H est factorisee sous la forme H = ZZ(transpose) * DV * ZZ
!		  ZZ : matrice orthogonale des vecteurs propres de H
!		  DV : matrice diagonale des valeurs propres de H 
!		
!--------------------------------------------------------------------------------------
	SUBROUTINE INITH3(NM,PARA)
	USE CONSTANTES,ONLY: NP
	USE HOPPING
	USE PARA_SIM
	IMPLICIT NONE
	REAL(8)				:: TX,TY,DNORM,DNORM1,PI,DTAU	! Var. locales
	REAL(8)				:: DKX,DKY,PHASE,QX,QY,ENGY	! Var. locales
	INTEGER				:: II,I,J,IK,KX,KY,KXRF,KYRF	! Var. locales
	INTEGER				:: IKRF,IX,IR,IY,ICOUNT		! Var. locales
	REAL(8),DIMENSION (NP)		:: DVN				! Var. locale
	INTEGER,INTENT(IN)		:: NM
	INTEGER,DIMENSION (NM)		:: I_X,I_Y			! Var. locales
	TYPE(PARAMETRE),INTENT(IN)	:: PARA
!
	DTAU=PARA%BETA/DBLE(PARA%TRAN)
	PI=DACOS(-1.D0)
	TX=1.D0
	IF(PARA%N==2) TX=0.5D0
	TY=1.D0
	IF(PARA%M==1) TY=0.D0
	IF(PARA%M==2) TY=0.5D0
	DNORM=DSQRT(2.D0)/DSQRT(DBLE(NM))
	DNORM1=1.D0/DSQRT(DBLE(NM))
	DKX=2.D0*PI/DBLE(PARA%N)
	DKY=2.D0*PI/DBLE(PARA%M)
	ICOUNT=0
!
!	Reperage cartesien
	II=0
	DO J=1,PARA%M
	   DO I=1,PARA%N
	      II=II+1
	      I_X(II)=I
	      I_Y(II)=J
	   END DO
	END DO
!	
!	Definition de la matrice de transformation ZZ
	DO IK=1,NM
	   KX=I_X(IK)
	   KY=I_Y(IK)
	   KXRF=PARA%N-KX
	   IF(KXRF==0) KXRF=PARA%N
	   KYRF=PARA%M-KY
	   IF(KYRF==0) KYRF=PARA%M
	   IKRF=(KYRF-1)*PARA%N+KXRF
	   IF(IKRF>IK) THEN
	      ICOUNT=ICOUNT+2
	      DO IR=1,NM
	         IX=I_X(IR)
	         IY=I_Y(IR)
	         PHASE=DKX*DBLE(IX*KX)+DKY*DBLE(IY*KY)
	         ZZ(ICOUNT,IR)=DNORM*DCOS(PHASE)
	         ZZ(ICOUNT-1,IR)=DNORM*DSIN(PHASE)
	      END DO
	      QX=DKX*DBLE(KX)
	      QY=DKY*DBLE(KY)
	      ENGY=2.D0*PARA%TPAR*TX*DCOS(QX)+2.D0*PARA%TPER*TY*DCOS(QY)+&
	      	  &4.D0*PARA%TPRM*TX*TY*DCOS(QX)*DCOS(QY)
	      DV(ICOUNT)=DEXP((ENGY+PARA%MU)*DTAU)
	      DV(ICOUNT-1)=DEXP((ENGY+PARA%MU)*DTAU)
	   ENDIF
	   IF(IKRF==IK) THEN
	      ICOUNT=ICOUNT+1
	      DO IR=1,NM
	         IX=I_X(IR)
	         IY=I_Y(IR)
	         PHASE=DKX*DBLE(IX*KX)+DKY*DBLE(IY*KY)
	         ZZ(ICOUNT,IR)=DNORM1*DCOS(PHASE)
	      END DO
	      QX=DKX*DBLE(KX)
	      QY=DKY*DBLE(KY)
	      ENGY=2.D0*PARA%TPAR*TX*DCOS(QX)+2.D0*PARA%TPER*TY*DCOS(QY)+&
	          &4.D0*PARA%TPRM*TX*TY*DCOS(QX)*DCOS(QY)
	      DV(ICOUNT)=DEXP((ENGY+PARA%MU)*DTAU)
	   ENDIF
	END DO
!
!	Calcul de H = exp(K) = ZZT*DV*ZZ
	CALL MAC10(NP,NM,ZZ,H)					! H<=ZZ
	CALL MAC11(NP,NM,H)					! H<=TRANSPOSE(ZZ)
	CALL MULD(NP,NM,2,H,DV)					! H<=H*DV
	CALL MAC2(NP,NM,H,ZZ)					! H<=H*ZZ	
!	
!	Calcul de HM = exp(-K) = ZZT*DV^-1*ZZ
	DO I=1,NM
	   DVN(I)=1/DV(I)
	END DO
	CALL MAC10(NP,NM,ZZ,HM)
	CALL MAC11(NP,NM,HM)
	CALL MULD(NP,NM,2,HM,DVN)
	CALL MAC2(NP,NM,HM,ZZ)	
!
!	Calcul de H2P = exp(K/2) = ZZT*SQRT(DV)*ZZ
	DO I=1,NM
	   DVN(I)=DSQRT(DV(I))
	END DO
	CALL MAC10(NP,NM,ZZ,H2P)
	CALL MAC11(NP,NM,H2P)
	CALL MULD(NP,NM,2,H2P,DVN)
	CALL MAC2(NP,NM,H2P,ZZ)
!
!	Calcul de H2M = exp(-K/2) = ZZT*SQRT(DV)^-1*ZZ	
	DO I=1,NM
	   DVN(I)=1/DSQRT(DV(I))
	END DO
	CALL MAC10(NP,NM,ZZ,H2M)
	CALL MAC11(NP,NM,H2M)
	CALL MULD(NP,NM,2,H2M,DVN)
	CALL MAC2(NP,NM,H2M,ZZ)
!	
	END SUBROUTINE INITH3
!--------------------------------------------------------------------------------------
! Nom		: INITV	[SUB]							15
! Fonction	: Initialisation des registres VUP,VDWN et SPIN (variables du champ HS)
! Auteurs	: AM.Tremblay,A.Reid,H.Nelisse,C.Bourbonnais,C.Boily,L.Chen,H.Touchette
! Revision	: 22 juillet 1997 (Touchette)
! Notes		: - ZETA(2)='y' ou 'Y' : Initialisation a partir d'une configuration
!		    archivee dans FICHSIN sinon, initialisation aleatoire.
!--------------------------------------------------------------------------------------
	SUBROUTINE INITV(PARA,DSEED,CTEU,CTED,SPIN,VUP,VDN)
	USE CONSTANTES,ONLY: NP,NL
	USE FICHIERS,ONLY: FICHSIN
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER						:: I,J,NM	! Var. locales
	REAL(8)						:: DTAU,DEX	! Var. locales
	REAL(8)						:: ASQT,XLAMBDA	! Var. locales
	REAL(8)						:: GGUBFS
	REAL(8),INTENT(OUT)				:: CTEU,CTED
	REAL(8),DIMENSION (NP,NL),INTENT(OUT)		:: VUP,VDN
	INTEGER,DIMENSION (NP,NL),INTENT(OUT)		:: SPIN
	INTEGER(SELECTED_INT_KIND(9)),INTENT(INOUT)	:: DSEED
	TYPE(PARAMETRE),INTENT(IN)			:: PARA
!
!	Initialisation de la configuration
	NM=PARA%N*PARA%M
	SELECT CASE(PARA%ZETA(2))
	   CASE('y','Y')				! Configuration archivee
	      OPEN(UNIT=5,FILE=FICHSIN,ERR=65)
	      REWIND(UNIT=5)
	      DO I=1,PARA%TRAN					
	         READ(5,*)(SPIN(J,I),J=1,NM)			
	      END DO
	      CLOSE(UNIT=5)
	   CASE DEFAULT					! Configuration aleatoire
	      DO I=1,PARA%TRAN
	         DO J=1,NM
	            IF(GGUBFS(DSEED).GE.0.5) THEN
	               SPIN(J,I)=1
	            ELSE
	               SPIN(J,I)=-1
	            ENDIF
	         END DO
	      END DO
	END SELECT
!
!	Initialisation des registres VUP et VDN
	DTAU=PARA%BETA/DBLE(PARA%TRAN)
	DEX=DEXP(0.5D0*DTAU*DABS(PARA%U))
	ASQT=DSQRT(DEX*DEX-1.D0)
	XLAMBDA=DLOG(DEX+ASQT)
	CTEU=DEXP(XLAMBDA)
	CTED=DEXP(-XLAMBDA)
	SELECT CASE(PARA%U.GT.0)
	   CASE(.TRUE.)						! U > 0						
	      DO I=1,PARA%TRAN
	         DO J=1,NM
	            IF(SPIN(J,I).EQ.1) THEN			! SPIN = 1
	               VUP(J,I)=CTEU
	               VDN(J,I)=CTED
	            ELSE					! SPIN =-1
	               VUP(J,I)=CTED
	               VDN(J,I)=CTEU
	            ENDIF
	         END DO
	      END DO
	   CASE(.FALSE.)					! U < 0
	      DO I=1,PARA%TRAN
	         DO J=1,NM
	            IF(SPIN(J,I).EQ.1) THEN			! SPIN = 1
	               VUP(J,I)=CTEU
	               VDN(J,I)=CTEU
	            ELSE					! SPIN =-1
	               VUP(J,I)=CTED
	               VDN(J,I)=CTED
	            ENDIF
	         END DO
	      END DO
	END SELECT
	RETURN
65	WRITE(*,*) '--- Erreur : Fichier inexistant (INITV) ---'
	STOP
	END SUBROUTINE INITV	
!--------------------------------------------------------------------------------------
! Nom		: MAC4251	[SUB]						16
! Fonction	: Multiplication par la gauche, la droite ou les deux (enroulement
!	 	  de la fonction de Green) au 1er ordre de la decomposition de Trotter. 
! Auteurs	: H.Touchette
! Revision	: 27 octobre 1996 (Touchette)
! Notes		: Utilise la forme dense de H sous sa forme factorisee H = ZZT DV ZZ
!		  (forme de Liang Chen)
!--------------------------------------------------------------------------------------
	SUBROUTINE MAC4251(NM,IST,G,V)
	USE CONSTANTES,ONLY: NP
	USE HOPPING
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NM,IST
	INTEGER					:: I
	REAL(8),DIMENSION (NP),INTENT(IN)	:: V
	REAL(8),DIMENSION (NP)			:: VN		! Variable locale	
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: G	
!
!	Produit par la droite - IST = 1 ou 2 : G'= G*(H*V)^-1
	IF(IST.NE.0) THEN
	   DO I=1,NM
	      VN(I)=1/V(I)					! VN=exp(-V)
	   END DO
	   CALL MULD(NP,NM,2,G,VN)				! G<=G*VN
	   CALL MAC2(NP,NM,G,HM)				! G<=G*exp(-K)
        ENDIF
!
!	Produit par la gauche - IST = 0 ou 1 : G'= (H*V)*G
        IF(IST.NE.2) THEN
	   CALL MULD(NP,NM,0,G,V)				! G<=V*G
	   CALL MAC1(NP,NM,H,G)					! G<=exp(K)*G
        ENDIF
!		
	END SUBROUTINE MAC4251
!--------------------------------------------------------------------------------------
! Nom		: INITW 	[SUB]						17
! Fonction	: Calcul des coefficients pour le l'integration 
! Auteurs	:
! Revision	: 1 decembre 1996 (Touchette)
! Notes		: - Simpson (integrale a 3 points) : ! S
!		  - Trapeze (integrale a 2 points) : ! T 
!		  - Noter la borne TRAN+1
!--------------------------------------------------------------------------------------	
	SUBROUTINE INITW(NTRAN)
	USE UTILITAIRE,ONLY: W
	IMPLICIT NONE
	INTEGER,INTENT(IN)		:: NTRAN
	INTEGER				:: I			! Variable locale
	W(1)=1.D0/3.D0						! S
!	W(1)=1.D0/2.D0						! T
	DO I=2,NTRAN
	   W(I)=4.D0/3.D0-2.D0/3.D0*MOD(I,2)			! S
!	   W(I)=1.D0						! T
	END DO
	W(NTRAN+1)=1.D0/3.D0					! S
!	W(NTRAN+1)=1.D0/2.D0					! T					
	END SUBROUTINE INITW
!--------------------------------------------------------------------------------------
! Nom		: LUDCMP 	[SUB]						18
! Fonction	: Decomposition gaussienne A = LU
! Auteurs	:
! Revision	:
! Notes		: Provient entierement de Numerical Recipes
!--------------------------------------------------------------------------------------	
      	SUBROUTINE LUDCMP(A,N,NP,INDX,D)
      	IMPLICIT REAL(8) (A-H,O-Z)
      	PARAMETER (NMAX=200,TINY=1.0E-20)
      	DIMENSION A(NP,NP),INDX(N),VV(NMAX)
      	D=1.D0
      	DO 12 I=1,N
           AAMAX=0.D0
           DO 11 J=1,N
              IF(ABS(A(I,J)).GT.AAMAX) AAMAX=ABS(A(I,J))
11         CONTINUE
           IF(AAMAX.EQ.0.) PAUSE '--- Erreur : Singular matrix LUDCMP'
           VV(I)=1.D0/AAMAX
12     	CONTINUE
      	DO 19 J=1,N
           IF(J.GT.1) THEN
              DO 14 I=1,J-1
                 SUM=A(I,J)
                 IF(I.GT.1) THEN
                    DO 13 K=1,I-1
                       SUM=SUM-A(I,K)*A(K,J)
13                  CONTINUE
                    A(I,J)=SUM
                 ENDIF
14            CONTINUE
           ENDIF
           AAMAX=0.D0
           DO 16 I=J,N
              SUM=A(I,J)
              IF(J.GT.1) THEN
                 DO 15 K=1,J-1
                    SUM=SUM-A(I,K)*A(K,J)
15               CONTINUE
                 A(I,J)=SUM
              ENDIF
              DUM=VV(I)*ABS(SUM)
              IF(DUM.GE.AAMAX) THEN
                 IMAX=I
                 AAMAX=DUM
              ENDIF
16         CONTINUE
           IF(J.NE.IMAX) THEN
              DO 17 K=1,N
                 DUM=A(IMAX,K)
                 A(IMAX,K)=A(J,K)
                 A(J,K)=DUM
17            CONTINUE
              D=-D
              VV(IMAX)=VV(J)
           ENDIF
           INDX(J)=IMAX
           IF(J.NE.N) THEN
             IF(A(J,J).EQ.0.) A(J,J)=TINY
             DUM=1./A(J,J)
             DO 18 I=J+1,N
                A(I,J)=A(I,J)*DUM
18           CONTINUE
           ENDIF
19    	CONTINUE
      	IF(A(N,N).EQ.0.) A(N,N)=TINY
      	RETURN
      	END
!--------------------------------------------------------------------------------------
! Nom		: INVMG 	[SUB]						19-(1)
! Fonction	: Inversion d'une matrice carree
! Auteurs	: H.Touchette
! Revision	: 31 juillet 97 (Touchette)
! Notes		: A -> A^(-1)=AI 
!		  Provient de Numerical Recipes, p.40.
! Dependances	: LUDCMP,LUBKSB (externes)
!--------------------------------------------------------------------------------------	
	SUBROUTINE INVMG(NP,NM,A)
	IMPLICIT NONE
	REAL(8)					:: D		! Var. locale
	INTEGER					:: I,J		! var. locale
	INTEGER,INTENT(IN)			:: NP,NM
	INTEGER,DIMENSION (NP)			:: INDX   	! Var. locale
	REAL(8),DIMENSION (NP,NP)		:: Y		! Var. locale
	REAL(8),DIMENSION (NP,NP),INTENT(INOUT)	:: A
!
	DO I=1,NM
	   DO J=1,NM
	      Y(I,J)=0.D0
	   END DO
	   Y(I,I)=1.D0
	END DO
	CALL LUDCMP(A,NM,NP,INDX,D)
	DO J=1,NM
	   CALL LUBKSB(A,NM,NP,INDX,Y(1,J))
	END DO
	CALL MAC10(NP,NM,Y,A)						
	END SUBROUTINE INVMG
!--------------------------------------------------------------------------------------
! Nom		: LUBKSB 	[SUB]						19-(2)
! Fonction	: Backsubstitution (pour l'inversion)
! Auteurs	:
! Revision	:
! Notes		: Provient de Numerical Recipes
!--------------------------------------------------------------------------------------	
	SUBROUTINE LUBKSB(A,N,NP,INDX,B)
	INTEGER N,NP,INDX(N)
	REAL(8) A(NP,NP),B(N)
	INTEGER I,II,J,LL
	REAL(8) SUM
	II=0
	DO I=1,N
	   LL=INDX(I)
	   SUM=B(LL)
	   B(LL)=B(I)
	   IF(II.NE.0) THEN
	      DO J=II,I-1
	         SUM=SUM-A(I,J)*B(J)
	      END DO
	   ELSE IF(SUM.NE.0) THEN
	      II=I
	   END IF
	   B(I)=SUM
	END DO
	DO I=N,1,-1
	   SUM=B(I)
	   DO J=I+1,N
	      SUM=SUM-A(I,J)*B(J)
	   END DO
	   B(I)=SUM/A(I,I)
	END DO
	RETURN
	END
!--------------------------------------------------------------------------------------
! Nom		: GGUBFS 	[FUNCTION]					20
! Fonction	: Generateur de nombres pseudo-aleatoires
! Auteurs	: 
! Revision	: 15 decembre 1996
! References	: [1] Numerical Recipes, Fortran 90, p.1142.
! Notes		: Remplace le generateur de P. L'Ecuyer.
!
!--------------------------------------------------------------------------------------
	FUNCTION GGUBFS(IDUM)
	IMPLICIT NONE
	INTEGER,PARAMETER 		:: K4B=SELECTED_INT_KIND(9)
	INTEGER(K4B),INTENT(INOUT) 	:: IDUM
	REAL(8) 			:: GGUBFS
	INTEGER(K4B),PARAMETER 		:: IA=16807,IM=2147483647,IQ=127773,IR=2836
	REAL(8),SAVE 			:: AM
	INTEGER(K4B),SAVE 		:: IX=-1,IY=-1,K
!
	IF(IDUM<=0.OR.IY<0) THEN
	   AM=NEAREST(1.0,-1.0)/IM
	   IY=IOR(IEOR(888889999,ABS(IDUM)),1)
	   IX=IEOR(777755555,ABS(IDUM))
	   IDUM=ABS(IDUM)+1
	ENDIF
	IX=IEOR(IX,ISHFT(IX,13))
	IX=IEOR(IX,ISHFT(IX,-17))
	IX=IEOR(IX,ISHFT(IX,5))
	K=IY/IQ
	IY=IA*(IY-K*IQ)-IR*K
	IF(IY<0) IY=IY+IM
	GGUBFS=AM*IOR(IAND(IM,IEOR(IX,IY)),1)
	END FUNCTION GGUBFS
!--------------------------------------------------------------------------------------
! Nom		: INIT_LOC 	[SUB]						21
! Fonction	: Initialisation des vecteurs de localisation des sites
! Auteurs	:
! Revision	: 22 octobre 1996 (Touchette)
! Notes		: Le site I se trouve sur CHI(I) et COLI(I)
!--------------------------------------------------------------------------------------
	SUBROUTINE INIT_LOC(N,M)
	USE UTILITAIRE,ONLY: CHI,COLI
	IMPLICIT NONE	
	INTEGER,INTENT(IN)		:: N,M	
	INTEGER				:: I,NM			! Variables locales
	NM=N*M
	DO I=1,NM
	   CHI(I)=(I+N-1)/N
	   COLI(I)=I-(CHI(I)-1)*N
	END DO
	END SUBROUTINE INIT_LOC
!--------------------------------------------------------------------------------------
! Nom		: DEFCOSMAG	[SUB]						22
! Fonction	: Construction de la matrice des cosinus pre-calcules
! Auteurs	: A.Veilleux,H.Touchette
! Revision	: 20 fevrier 1996 (Touchette)
! Notes		: Utilise par MAGST,PSCOR. Capacite fixee par ND1 et ND2
!--------------------------------------------------------------------------------------
	SUBROUTINE DEFCOSMAG(N,M)
	USE UTILITAIRE,ONLY: CHI,COLI,COSMAG
	IMPLICIT NONE
	REAL(8)				:: PHI,PI		! Variables locales
	INTEGER,INTENT(IN)		:: N,M
	INTEGER				:: KX,KY,I,J,NX,NY	! Variables locales
	INTEGER				:: NM,NCHI,NCHJ		! Variables locales
	INTEGER				:: NCOLJ,NCOLI		! Variables locales
!
	NM=N*M
	COSMAG(1:N,1:M,1:NM,1:NM)=0.D0				! Boucle implicite
	PI=DACOS(-1.D0)
	DO KX=1,N
	   DO KY=1,M
	     DO I=1,NM
	        NCHI=CHI(I)
	        NCOLI=COLI(I)
	        DO J=1,NM
	           NCHJ=CHI(J)
	           NCOLJ=COLI(J)
	           NX=NCOLI-NCOLJ
	           NY=NCHI-NCHJ
	           PHI=DCOS(2.D0*PI*KX/N*NX+2.D0*PI*KY/M*NY)
	           COSMAG(KX,KY,I,J)=PHI
	        END DO
	     END DO
	   END DO
	END DO
	END SUBROUTINE DEFCOSMAG
!--------------------------------------------------------------------------------------
! Nom		: AUTO_CORRELATION	[SUB]					23
! Fonction	: Calcul de la fonction d'autocorrelation
! Auteurs	: H.Touchette
! Revision	: 27 novembre 1996 (Touchette)
! References	: [1] Negele & Orland, Quantum Many-Particle Systems, Addison-Wesley,
!		      p.445.
!		  [2] Lacaze et al., cond-mat/9610193 28 octobre 1996.
!		  [3] Koonin & Meredith, Computational Physics, Addison-Wesley, 1989.
! Notes		: - Calcul seulement a la fin d'une simulation
!		  - Les donnees dans AUTOCORR sont remplacees a la sortie par la
!		    fonction d'autocorrelation 
!--------------------------------------------------------------------------------------
	SUBROUTINE AUTO_CORRELATION(C1,AUTOCORR)
	USE CONSTANTES,ONLY: NCOR
	IMPLICIT NONE	
	INTEGER					:: MC1,I,K		! Var locales
	REAL(8)					:: SUM,MOY1,MOY2	! var locales
	REAL(8),INTENT(IN)			:: C1
	REAL(8),DIMENSION (NCOR)		:: TEMP			! Var locale		
	REAL(8),DIMENSION (NCOR),INTENT(INOUT)	:: AUTOCORR
!
	MC1=INT(C1)
	MOY1=0.D0
	MOY2=0.D0
	DO I=1,MC1
	   MOY1=MOY1+AUTOCORR(I)
	   MOY2=MOY2+AUTOCORR(I)*AUTOCORR(I)
	END DO
	MOY1=(MOY1/C1)**2
	MOY2=(MOY2/C1)-MOY1					
	DO K=0,MC1-1
	   SUM=0.D0
	   DO I=1,MC1-K
	      SUM=SUM+AUTOCORR(I)*AUTOCORR(I+K)
	   END DO
	   SUM=SUM/DBLE(MC1-K)						
	   TEMP(K+1)=(SUM-MOY1)/MOY2
	END DO	
	AUTOCORR(1:MC1)=TEMP(1:MC1)
	END SUBROUTINE AUTO_CORRELATION
!--------------------------------------------------------------------------------------
! Nom		: IDENTIFICATION	[SUB]					24
! Fonction	: Initialisation des parametres d'identification des fichiers
! Auteurs	: H.Touchette
! Revision	: 15 juillet 1997 (Touchette)
! References	: [1] AIX XL Fortran Compiler/6000, Language Reference, p.33-
! Notes		: 
!--------------------------------------------------------------------------------------
	SUBROUTINE IDENTIFICATION(PARA,DSEEDI)
	USE FICHIERS,ONLY: FICPAOUT,FICHSOUT,FICGIOUT,FICCVOUT,REPWEB
	USE IDENT
	USE PARA_SIM
	IMPLICIT NONE
	CHARACTER(LEN=10)				:: NUMTOASCII
	CHARACTER(LEN=NCA)				:: TP1,TP2,TP3,TP4,TPI	! Loc.
	INTEGER						:: I1,I2,I3,I4,I5,I6,II	! Loc.
	INTEGER(SELECTED_INT_KIND(9)),INTENT(IN)	:: DSEEDI
	TYPE(PARAMETRE),INTENT(IN)			:: PARA
!
	IDNUM=DSEEDI
	TP1=NUMTOASCII(INT(DABS(PARA%U)))
	TP2=NUMTOASCII(PARA%N)
	TP3=NUMTOASCII(PARA%M)
	TP4=NUMTOASCII(INT(PARA%BETA))
	TPI=NUMTOASCII(IDNUM)
	I1=INDEX(TP1,' ')
	I2=INDEX(TP2,' ')
	I3=INDEX(TP3,' ')
	I4=INDEX(TP4,' ')
	II=INDEX(TPI,' ')
	I5=INDEX(PREFIX,' ')
	I6=INDEX(SUFFIX,' ')
	NANUM=PREFIX(1:I5-1)//TP1(1:I1-1)//TP2(1:I2-1)//TP3(1:I3-1)//TP4(1:I4-1)
	I5=INDEX(NANUM,' ')
	FICPAOUT=TPI(1:II-1)//'.'//NANUM(1:I5-1)//'re'//'.'//SUFFIX(1:I6-1)
	FICHSOUT=TPI(1:II-1)//'.'//NANUM(1:I5-1)//'hs'//'.'//SUFFIX(1:I6-1)
	FICGIOUT=TPI(1:II-1)//'.'//NANUM(1:I5-1)//'gi'//'.'//SUFFIX(1:I6-1)
	FICCVOUT=TPI(1:II-1)//'.'//NANUM(1:I5-1)//'cv'//'.'//SUFFIX(1:I6-1)
	END SUBROUTINE IDENTIFICATION
!--------------------------------------------------------------------------------------
! Nom		: NUMTOASCII	[FUNCTION]					25
! Fonction	: Cree des suffixes numeraux a partir de la liste ascii de 0 a 9
! Auteurs	: C.Foisy
! Revision	: 21 juillet 1997 (Touchette)
! References	: [1] AIX XL Fortran Compiler/6000, Language Reference, p.33-
! Notes		: 
!--------------------------------------------------------------------------------------
	FUNCTION NUMTOASCII(I)
	USE CONSTANTES,ONLY: NCA
	IMPLICIT NONE
	CHARACTER(LEN=10)			:: NUMTOASCII
	CHARACTER(LEN=10)			:: TMP		! Variable locale
	INTEGER,INTENT(IN)			:: I
	INTEGER					:: J,K		! Variables locales
!
	J=I
	K=1
	TMP(1:10)='          '
	NUMTOASCII(1:10)='          '
	DO WHILE(J.GT.0.OR.K.EQ.1)
	   TMP(K:K)=CHAR(ICHAR('0')+MOD(J,10))
	   K=K+1
	   J=J/10
	END DO
	J=INDEX(TMP,' ')
	DO K=1,J-1
	   NUMTOASCII(K:K)=TMP(J-K:J-K)
	END DO
	END FUNCTION NUMTOASCII
!--------------------------------------------------------------------------------------
! Nom		: GLOBAL_MOVE	[SUB]						26
! Fonction	: Basculement des spins d'un site sur toutes les tranches de temps
! Auteurs	: H.Touchette
! Revision	: 6 mai 1997 (Touchette)
! References	: [1] Scalettar et al.,PRB,44,10502,1991.
! Notes		: 
!--------------------------------------------------------------------------------------
	SUBROUTINE GLOBAL_MOVE(NTRAN,CUP,CDN,DU,SPN,VUP,VDN,NCH)
	USE CONSTANTES,ONLY: NP,NL
	INTEGER						:: I		! Var. locale
	INTEGER,INTENT(IN)				:: NTRAN,NCH		
	REAL(8),INTENT(IN)				:: CUP,CDN,DU
	INTEGER,DIMENSION(NP,NL),INTENT(INOUT)		:: SPN
	REAL(8),DIMENSION(NP,NL),INTENT(INOUT)		:: VUP,VDN
!
	IF(DU.GT.0) THEN
	   DO I=1,NTRAN 
	      IF(SPN(NCH,I).EQ.1) THEN
	         VUP(NCH,I)=CDN
	         VDN(NCH,I)=CUP
	         SPN(NCH,I)=-1
	      ELSE
	         VUP(NCH,I)=CUP
	         VDN(NCH,I)=CDN
	         SPN(NCH,I)=1
	      ENDIF
	   END DO
	ELSE
	   DO I=1,NTRAN 
	      IF(SPN(NCH,I).EQ.1) THEN
	         VUP(NCH,I)=CDN
	         VDN(NCH,I)=CDN
	         SPN(NCH,I)=-1
	      ELSE
	         VUP(NCH,I)=CUP
	         VDN(NCH,I)=CUP
	         SPN(NCH,I)=1
	      ENDIF
	   END DO
	ENDIF
	END SUBROUTINE GLOBAL_MOVE
!--------------------------------------------------------------------------------------
!			    	  ROUTINES - TEMPS EGAL
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
! Nom		: GCALS 	[SUB]						ME1
! Fonction	: Calcul de la fonction de Green a temps egal
! Auteurs	: AM.Tremblay,A.Reid,H.Nelisse,C.Bourbonnais,C.Boily,H.Touchette
! Revision	: 28 juillet 1997 (Touchette)
! Reference 	: H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		  hiver 96 - ete 97.
! Notes		: - Avec procedure de stabilisation (orthogonalisation + pivot)
!		  - Calcul optionnel du determinant de G^-1 
!--------------------------------------------------------------------------------------
	SUBROUTINE GCALS(NM,L,PARA,V,G,DET)
	USE CONSTANTES,ONLY: NP,NL
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER					:: I,J,K	! Var. locales
	REAL(8)					:: DETD,DETR 	! Var. locales
	INTEGER,INTENT(IN)			:: NM,L
	REAL(8),INTENT(OUT)			:: DET
	INTEGER,DIMENSION (NP)			:: INDX		! Var. locale
	REAL(8),DIMENSION (NP)			:: D		! Var. locale
	REAL(8),DIMENSION (NP,NP)		:: U,R,R2,R3	! Var. locales
	REAL(8),DIMENSION (NP,NP),INTENT(OUT)	:: G
	REAL(8),DIMENSION (NP,NL),INTENT(IN)	:: V
	TYPE(PARAMETRE),INTENT(IN)		:: PARA
!
!	Initialisation des matrices
	DO J=1,NM
	   DO I=1,NM
	      U(I,J)=0.D0							
	      R(I,J)=0.D0							
	   END DO
	   U(J,J)=1.D0
	   R(J,J)=1.D0
	   D(J)=1.D0
	END DO
!
!	Boucle de L a TRAN
	K=L
	DO WHILE(K.LT.PARA%TRAN+1)
	   DO I=1,PARA%NMODUL					! U=B(K+NMODUL)...B(K)
	      CALL MAC4251(NM,0,U,V(1,K))						
	      K=K+1
	   END DO
	   CALL MULD(NP,NM,2,U,D)				! U = U*D
	   CALL GRASCHP(NP,NM,U,D,R2)				! UDV (V=R2) 
	   CALL MAC1(NP,NM,R2,R)				! V finale = R
	END DO
!
!	Boucle de 1 a L-1 
	K=1
	DO WHILE(K.LT.L)			
	   DO I=1,PARA%NMODUL
	      CALL MAC4251(NM,0,U,V(1,K))
	      K=K+1
	   END DO
	   CALL MULD(NP,NM,2,U,D)
	   CALL GRASCHP(NP,NM,U,D,R2)
	   CALL MAC1(NP,NM,R2,R)
	END DO
!
!	Decomposition UDV du produit des matrices B 
!	                 -1  -1 -1  -1 -1
!	Calcul de   G = V  (U  V +D)  U
!
!	           -1 -1
!	Calcul de U  V  +D
!
	CALL MAC10(NP,NM,R,R3)					! V													
	CALL INVMG(NP,NM,R)					! V^-1					
 	CALL MAC10(NP,NM,U,G)													         
	CALL MAC11(NP,NM,U)					! U <= U^T = U^-1					
	CALL MAC2(NP,NM,U,R)					! U <= U^-1 V^-1	
	DO I=1,NM					
	   U(I,I)=U(I,I)+D(I)					! U <= U^-1 V^-1 +D
	END DO
!
!	Decomposition UDV avant inversion				      -1 -1
	CALL GRASCHP(NP,NM,U,D,R2)				! U'D'V' <= (U  V  +D)
!
!	Calcul optionnel du determinant 
	IF((PARA%ZETA(4).EQ.'Y'.OR.PARA%ZETA(4).EQ.'y').AND.L.EQ.1) THEN
	   DETD=1.D0
	   DO I=1,NM
	      DETD=DETD*D(I)
	   END DO
	   CALL MAC1(NP,NM,R2,R3)				
	   CALL LUDCMP(R3,NM,NP,INDX,DETR)
	   DO I=1,NM
	      DETR=DETR*R3(I,I)
	   END DO
	   DET=DETR*DETD
	ENDIF
!
!                  -1  -1 -1      -1
!	Inversion V  R2  D  (U3 U)						   
	CALL INVMG(NP,NM,R2)					! V'^-1
	CALL MAC1(NP,NM,R,R2)					! V^-1 * V'^-1						
	DO J=1,NM						!        								
	   DO I=1,NM						! (V^-1 V'^-1)*D^-1						
	      R2(I,J)=R2(I,J)/D(J)
	   END DO
	END DO
	CALL MAC2(NP,NM,G,U)					! U U'									   
	CALL MAC11(NP,NM,G)					! G <= (U U')^T
	CALL MAC1(NP,NM,R2,G)					! V2*G						
	END SUBROUTINE
!--------------------------------------------------------------------------------------
! Nom		: OCC_REMP		[SUB]					ME2
! Fonction	: Calcul des parametres de la bande (remplissage, mu propose)
! Auteurs	: 
! Revision	: 6 novembre 1996 (Touchette)
! Notes		: 
!--------------------------------------------------------------------------------------
	SUBROUTINE OCC_REMP(NP,NM,SIG,GUP,GDWN,DMU,SOCC,XBETA)
	IMPLICIT NONE
	INTEGER					:: I,J			! Var. locales
	REAL(8)					:: SOM,SI,SJ,SUP,SDW	! Var. locales
	INTEGER,INTENT(IN)			:: NP,NM
	REAL(8),INTENT(INOUT)			:: DMU
	REAL(8),INTENT(OUT)			:: SOCC
	REAL(8),INTENT(IN)			:: XBETA,SIG
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: GUP,GDWN
!
	SOM=0.D0
	DO I=1,NM
	   SOM=SOM+2.D0-GUP(I,I)-GDWN(I,I)
	END DO
	SOCC=SOM/NM
	SOM=0.D0
	DO I=1,NM
	   SI=2.D0-GUP(I,I)-GDWN(I,I)
	   DO J=1,NM
	      SJ=2.D0-GUP(J,J)-GDWN(J,J)
	      SUP=GUP(J,I)*GUP(I,J)
	      SDW=GDWN(J,I)*GDWN(I,J)
	      SOM=SOM+SI*SJ-SUP-SDW
	   END DO
	   SOM=SOM+GUP(I,I)+GDWN(I,I)
	END DO
	DMU=DMU-SIG*XBETA*SOM/NM
	END SUBROUTINE OCC_REMP
!--------------------------------------------------------------------------------------
! Nom		: ENETOT	[SUB]						ME3
! Fonction	: Calcul de l'energie totale des electrons libres ayant le meme
!		  potentiel chimique mu
! Auteurs	: AM.Tremblay
! Revision	: 27 octobre 1996 (Touchette)
! Notes		:
!--------------------------------------------------------------------------------------
	SUBROUTINE ENE_TOT(PARA,ENERG)
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER					:: I,J,NM		! Var. locales
	REAL(8),INTENT(OUT)			:: ENERG		
	REAL(8) 				:: KX,KY		! Var. locales
	REAL(8)					:: PI,EPS,DEN,ENE
	TYPE(PARAMETRE),INTENT(IN)		:: PARA
!
	PI=DACOS(-1.D0)
	NM=PARA%N*PARA%M
	ENERG=0.D0
	IF(PARA%M.NE.1) THEN
	   DO I=0,PARA%N-1
	      KX=2.D0*PI*DBLE(I)/DBLE(PARA%N)
	      DO J=0,PARA%M-1
	         KY=2.D0*PI*DBLE(J)/DBLE(PARA%M)
	         EPS=ENE(PARA%N,PARA%M,PARA%TPAR,PARA%TPER,PARA%TPRM,KX,KY)
	         DEN=DEXP(PARA%BETA*(EPS-PARA%MU))+1
	         ENERG=ENERG+EPS/DEN
	      END DO
	   END DO
	ELSE
	   DO I=0,PARA%N-1
	      KX=2.D0*PI*DBLE(I)/DBLE(PARA%N)
	      EPS=ENE(PARA%N,PARA%M,PARA%TPAR,PARA%TPER,PARA%TPRM,KX,KY)
	      DEN=DEXP(PARA%BETA*(EPS-PARA%MU))+1
	      ENERG=ENERG+EPS/DEN
	   END DO
	ENDIF
	ENERG=ENERG*2.D0/DBLE(NM)
	END SUBROUTINE ENE_TOT
!--------------------------------------------------------------------------------------
! Nom		: ENE		[FUNCTION]					ME4
! Fonction	: Relation de dispersion des electrons libres
! Auteurs	: H.Touchette
! Revision	: 15 juillet 1997 (Touchette)
! Notes		: - Tient compte des cas speciaux N,M = 2, de la periodicite du reseau,
!		    de meme que des seconds voisins.
!
!--------------------------------------------------------------------------------------	
	FUNCTION ENE(N,M,TPAR,TPER,TPRM,KX,KY)
	REAL(8)					:: ENE
	INTEGER,INTENT(IN)			:: N,M
	REAL(8),INTENT(IN)			:: TPAR,TPER,TPRM,KX
	REAL(8),INTENT(IN),OPTIONAL		:: KY
	REAL(8)					:: TPA,TPE,TP2		! Var. locales
!
	TPA=TPAR
	TPE=TPER
	TP2=TPRM
	IF(N.EQ.2) THEN
	   TPA=TPA/2
	   TP2=TP2/2
	ENDIF
	IF(M.EQ.2) THEN
	   TPE=TPE/2
	   TP2=TP2/2
	ENDIF
	IF(M.EQ.1) THEN
	   ENE=-2.D0*TPA*DCOS(KX)
	ELSE
	   ENE=-2.D0*(TPA*DCOS(KX)+TPE*DCOS(KY))-4*TP2*DCOS(KX)*DCOS(KY)
	ENDIF
	END FUNCTION ENE	 
!--------------------------------------------------------------------------------------
! Nom		: DSITE		[SUB]						ME5
! Fonction	: Calcul de l'energie 
! Auteurs	: AM.Tremblay,A.Reid,C.Boily
! Revision	: 21 juillet 1997 (Touchette)
! Reference	: [1] AM.Tremblay, Code Hirsch,Departement de physique de l'Universite
!		      de Sherbrooke.
! Notes		: - A cause des conditions periodiques, le cas 2 sites est special
!		  - Pas d'extension pour t' !$
!--------------------------------------------------------------------------------------
	SUBROUTINE DSITE(PARA,GUP,GDN,CIN,TOT)
	USE CONSTANTES,ONLY: NP
	USE UTILITAIRE,ONLY: CHI,COLI
	USE PARA_SIM
	IMPLICIT NONE
	REAL(8)					:: CTE,POT,CIN1,CIN2	! Var. locales
	REAL(8)					:: TPA,TPE,TP2		! Var. locales
	INTEGER					:: NM,I,II,II2		! Var. locales
	INTEGER					:: NCHI,NCOLI		! Var. locales
	REAL(8),INTENT(OUT)			:: CIN,TOT
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: GUP,GDN	
	REAL(8),DIMENSION (NP,NP)		:: BUP,BDN		! Var. locales
	TYPE(PARAMETRE),INTENT(IN)		:: PARA
!
	CTE=-1.D0
	NM=PARA%M*PARA%N
	CALL MAC8(NP,NM,GUP,BUP,CTE)
	CALL MAC8(NP,NM,GDN,BDN,CTE)
	CALL MAC9(NP,NM,BUP)					! BUP = 1 - GUP
	CALL MAC9(NP,NM,BDN)					! BDN = 1 - GDN
!
	CIN=0.D0
	POT=0.D0
	TOT=0.D0
	TPA=PARA%TPAR
	TPE=PARA%TPER
	TP2=PARA%TPRM
	IF(PARA%N.EQ.2) TPA=TPA/2.D0
	IF(PARA%M.EQ.2) TPE=TPE/2.D0
	DO I=1,NM
	   NCHI=CHI(I)
	   NCOLI=COLI(I)
	   II=I+1
	   II2=I-PARA%N
	   IF(NCOLI.EQ.PARA%N) II=I-PARA%N+1
	   IF(NCHI.EQ.1) II2=NM-PARA%N+I
	   CIN1=BUP(II,I)+BUP(I,II)+BDN(II,I)+BDN(I,II)
	   CIN2=BUP(II2,I)+BUP(I,II2)+BDN(II2,I)+BDN(I,II2)
	   IF(PARA%M.EQ.1) CIN2=0.D0
	   CIN=CIN+TPA*CIN1+TPE*CIN2
	   POT=POT+PARA%U*BUP(I,I)*BDN(I,I)
        END DO
	CIN=-CIN
	TOT=POT+CIN
	END SUBROUTINE DSITE
!--------------------------------------------------------------------------------------
! Nom		: DENSITE		[SUB]					ME6
! Fonction	: Calcul de l'impulsion 
! Auteurs	: AM.Tremblay,A.Reid,C.Boily
! Revision	: 21 juillet 1997 (Touchette)
! Reference	: [1] AM.Tremblay, Code Hirsch,Departement de physique de l'Universite
!		      de Sherbrooke.
! Notes		: 
!--------------------------------------------------------------------------------------
	SUBROUTINE DENSITE(N,M,GUP,GDN,X)
	USE CONSTANTES,ONLY: NP,ND1,ND2
	USE UTILITAIRE,ONLY: COSMAG
	IMPLICIT NONE
	REAL(8)					:: SUM,CTE		! Var. locales
	REAL(8)					:: GSU,GSD		! Var. locales
	INTEGER					:: I,J,KX,KY,NM		! Var. locales
	INTEGER,INTENT(IN)			:: N,M
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: GUP,GDN
	REAL(8),DIMENSION (NP,NP)		:: BUP,BDN		! Var. locales
	REAL(8),DIMENSION (ND1,ND2),INTENT(OUT)	:: X
!	
	NM=N*M
	CTE=-1.D0
	CALL MAC8(NP,NM,GUP,BUP,CTE)
	CALL MAC8(NP,NM,GDN,BDN,CTE)
	CALL MAC9(NP,NM,BUP)					! BUP = 1 - GUP
	CALL MAC9(NP,NM,BDN)					! BDN = 1 - GDN
	DO KX=1,N
	   DO KY=1,M
	      SUM=0.D0
	      DO I=1,NM
	         DO J=1,NM
	            GSU=BUP(I,J)+BUP(J,I)
	            GSD=BDN(I,J)+BDN(J,I)
	            SUM=SUM+COSMAG(KX,KY,I,J)*(GSU+GSD)
	         END DO
	      END DO
	      X(KX,KY)=SUM/NM
	   END DO
	END DO   
	END SUBROUTINE DENSITE
!--------------------------------------------------------------------------------------
! Nom		: PSCOR		[SUB]						ME7
! Fonction	: Calcul de Ps(Qx,Qy) : s-wave pair-field correlation function 
!		  (facteur de structure de paire)
! Auteurs	: AM.Tremblay,H.Touchette (1996)
! Revision	: Fevrier 1996 (Touchette)
! Reference	: [1] A.Moreo,D.J.Scalapino,PRB 45,7544,1992.
!		  [2] H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		      hiver 96 - ete 97.
! Notes		: 
!--------------------------------------------------------------------------------------
	SUBROUTINE PSCOR(N,M,GUP,GDN,X)
	USE CONSTANTES,ONLY: NP,ND1,ND2
	USE UTILITAIRE,ONLY: COSMAG
	IMPLICIT NONE
	REAL(8)					:: GIJ,SUM		! Var. locales
	REAL(8)					:: CTE,PHI		! Var. locales
	INTEGER					:: KX,KY,I,J		! Var. locales
	INTEGER					:: NM			! Var. locales
	INTEGER,INTENT(IN)			:: N,M
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: GUP,GDN
	REAL(8),DIMENSION (NP,NP)		:: AUP,ADW		! Var. locales
	REAL(8),DIMENSION (ND1,ND2),INTENT(OUT)	:: X
!
	NM=N*M
	CTE=-1.D0
	CALL MAC8(NP,NM,GUP,AUP,CTE)
	CALL MAC8(NP,NM,GDN,ADW,CTE)
	CALL MAC9(NP,NM,AUP)					! AUP = 1 - GUP
	CALL MAC9(NP,NM,ADW)					! ADW = 1 - GDWN
	DO KX=1,N
	   DO KY=1,M
	      SUM=0.D0
	      DO I=1,NM
	         DO J=1,NM
	            PHI=COSMAG(KX,KY,I,J)
	      	    GIJ=AUP(J,I)*ADW(J,I)+GUP(I,J)*GDN(I,J)
	            SUM=SUM+PHI*GIJ
	         END DO
	      END DO
	      X(KX,KY)=SUM/NM
	   END DO
	END DO
	END SUBROUTINE PSCOR
!--------------------------------------------------------------------------------------
! Nom		: MAGST		[SUB]						ME8
! Fonction	: Calcul du facteur de structure magnetique 
! Auteurs	: AM.Tremblay,A.Reid,H.Nelisse,C.Bourbonnais,C.Boily
! Revision	: 6 fevrier 1996 (Touchette)
! References	: [1] J.E.Hirsch,PRB 31,4409,1985.
!		  [2] Loh,Gubernatis pour le meilleur estimateur.
! Notes		: - Choix de l'estimateur [S+,S-] et [Sz], On active l'estimateur
! 	 	    voulu en enlevant ses marques de commentaires et en placant
! 	 	    des marques de commentaires pour l'autre estimateur.
!
!--------------------------------------------------------------------------------------
	SUBROUTINE MAGST(N,M,GUP,GDN,X)
	USE CONSTANTES,ONLY: NP,ND1,ND2
	USE UTILITAIRE,ONLY: COSMAG
	IMPLICIT NONE
	REAL(8)					:: PHI,CTE,SUM	! Variables locales
	REAL(8)					:: GI,GJ,GIJ	! Variables locales
	INTEGER					:: I,J,KX,KY,NM	! Variables locales
	INTEGER,INTENT(IN)			:: N,M
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: GUP,GDN
	REAL(8),DIMENSION (NP,NP)		:: AUP,ADW 	! Variables locales
	REAL(8),DIMENSION (ND1,ND2),INTENT(OUT)	:: X	
!
	NM=N*M
	CTE=-1.D0
        CALL MAC8(NP,NM,GUP,AUP,CTE)
	CALL MAC8(NP,NM,GDN,ADW,CTE)
        CALL MAC9(NP,NM,AUP)					! AUP = 1 - GUP	
        CALL MAC9(NP,NM,ADW) 					! ADW = 1 - GDWN  
!
	DO KX=1,N
	   DO KY=1,M
	   SUM=0.D0
	      DO I=1,NM
	         ! Utilise pour l'estimateur Sz (et non pour S+,S-)
                 GI=AUP(I,I)-ADW(I,I)
	         DO J=1,NM
	            ! Calcul de l'estimateur Sz (les 2 lignes qui suivent)
                    GJ=AUP(J,J)-ADW(J,J)
                    GIJ=GI*GJ+AUP(J,I)*GUP(I,J)+ADW(J,I)*GDN(I,J)
	            ! Calcul de l'estimateur S+,S-
	            ! GIJ=AUP(J,I)*GDN(I,J)+ADW(J,I)*GUP(I,J)
		    PHI=COSMAG(KX,KY,I,J)
		    SUM=SUM+PHI*GIJ
	         END DO
	      END DO
	      X(KX,KY)=SUM/NM
	   END DO
	END DO
	END SUBROUTINE MAGST
!--------------------------------------------------------------------------------------
! Nom		: RHOE		[SUB]						ME9
! Fonction	: Susceptibilite de charge a temps egal
! Auteurs	: H.Touchette
! Revision	: 16 juillet 1997 (Touchette)
! References	: H.Touchette, Aspects numeriques des simulation Monte Carlo quantique 
!		  du modele de Hubbard, rapport technique du CRPS, departement de 
!		  physique, UdeS, ete 1997.
! Notes		: - C(Qx,Qy)
!--------------------------------------------------------------------------------------
	SUBROUTINE RHOE(N,M,GUP,GDN,FST)
	USE CONSTANTES,ONLY: NP,NL,ND1,ND2
	USE UTILITAIRE,ONLY: COSMAG
	IMPLICIT NONE
	INTEGER					:: I,J,KX,KY,NM		! Var. locales
	REAL(8)					:: G1,G2,CTE		! Var. locales
	REAL(8)					:: PROD,PHI,ACC		! Var. locales
	INTEGER,INTENT(IN)			:: N,M
	REAL(8),DIMENSION (NP,NP)		:: AUP,ADN		! Var. locales
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: GUP,GDN
	REAL(8),DIMENSION (ND1,ND2),INTENT(OUT)	:: FST
!
	NM=N*M
	CTE=-1.D0
        CALL MAC8(NP,NM,GUP,AUP,CTE)
	CALL MAC8(NP,NM,GDN,ADN,CTE)
        CALL MAC9(NP,NM,AUP)					! AUP = 1 - GUP	
        CALL MAC9(NP,NM,ADN) 					! ADW = 1 - GDWN 
! 	
	DO KX=1,N
	   DO KY=1,M
	      ACC=0.D0
	      DO I=1,NM
	         G1=AUP(I,I)+ADN(I,I)
	         DO J=1,NM
	            G2=AUP(J,J)+ADN(J,J)
	            PROD=G1*G2+AUP(J,I)*GUP(I,J)+ADN(J,I)*GDN(I,J)
	            PHI=COSMAG(KX,KY,I,J)
	            ACC=ACC+PROD*PHI
	         END DO
	      END DO
	      FST(KX,KY)=ACC/NM
	   END DO
	END DO
	END SUBROUTINE RHOE
!--------------------------------------------------------------------------------------
!		 	   	ROUTINES - TEMPS INEGAL
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
! Nom		: GICALS	[SUB]						MI1
! Fonction	: Calcul des fonctions de Green a temps inegal au second ordre
!		  avec procedure de stabilisation
! Auteurs	: H.Touchette (1996)
! Revision	: 30 juillet 1997 (Touchette)
! Reference	: [1] H.Touchette, Aspects numeriques des simulations Monte Carlo  
!		      quantique du modele de Hubbard, rapport technique du CRPS,  
!		      departement de physique, UdeS, ete 1997.
!		  [2] H.G.Matuttis,Ph.D. Thesis,University of Regensburg,1995.
! Notes		: - En entree, on recoit G(l=1,l=1) au 1er ordre mais on calcule
!		    G(tau) au 2e ordre par MACGD
!		  
!--------------------------------------------------------------------------------------
	SUBROUTINE GICALS(NM,PARA,G,V,GI)
	USE CONSTANTES,ONLY: NP,NL
	USE PARA_SIM
	IMPLICIT NONE
	REAL*8						:: CTE
	INTEGER						:: I,J,K,NTOP	! Var. locales
	INTEGER,INTENT(IN)				:: NM
	REAL*8,DIMENSION (NP)				:: D,D2		! Var. locale
	REAL*8,DIMENSION (NP,NP),INTENT(IN)		:: G
	REAL*8,DIMENSION (NP,NP)			:: GP,GPP,AP	! Var. locales
	REAL*8,DIMENSION (NP,NP)			:: U,R,R2	! Var. locales 
	REAL*8,DIMENSION (NP,NL),INTENT(IN)		:: V
	REAL*8,DIMENSION(0:NL,NP,NP),INTENT(OUT)	:: GI
	TYPE(PARAMETRE),INTENT(IN)			:: PARA
!
	CTE=-1.D0
	CALL MACGD(NP,NM,G,GI(0,:,:))
	CALL MAC8(NP,NM,G,AP,CTE)				! AP<=-G
	CALL MAC9(NP,NM,AP)					! AP<=1-G
	CALL MACGD(NP,NM,AP,GI(PARA%TRAN,:,:))
!
	DO J=1,NM
	   DO K=1,NM
	      U(K,J)=0.D0
	      R(K,J)=0.D0
	   END DO
	   U(J,J)=1.D0
	   R(J,J)=1.D0
	   D(J)=1.D0
	END DO
!
	DO I=1,PARA%TRAN/2
	   CALL MAC4251(NM,0,U,V(:,I))
	   NTOP=MOD(I,PARA%NMODUL)
	   IF(NTOP.EQ.0) THEN
	      CALL MULD(NP,NM,2,U,D)
	      CALL GRASCHP(NP,NM,U,D,R2)
	      CALL MAC1(NP,NM,R2,R)
	   ENDIF
!             
	   CALL MAC10(NP,NM,G,GP)	
	   CALL MAC1(NP,NM,R,GP)
	   CALL MULD(NP,NM,0,GP,D)
	   CALL MAC1(NP,NM,U,GP)
	   CALL MACGD(NP,NM,GP,GI(I,:,:))
!
	   CALL MAC10(NP,NM,R,GP)
	   CALL INVMG(NP,NM,GP)
	   CALL MAC1(NP,NM,AP,GP)
	   DO J=1,NM
	      D2(J)=1.D0/D(J)
	   END DO
	   CALL MULD(NP,NM,2,GP,D2)
	   CALL MAC10(NP,NM,U,GPP)
	   IF(NTOP.EQ.0) THEN
	      CALL MAC11(NP,NM,GPP)
	   ELSE
	      CALL INVMG(NP,NM,GPP)
	   ENDIF
	   CALL MAC1(NP,NM,GP,GPP)
	   CALL MACGD(NP,NM,GPP,GI(PARA%TRAN-I,:,:)) 			
	END DO
!
	END SUBROUTINE GICALS
!--------------------------------------------------------------------------------------
! Nom		: MACGD		[SUB]						MI2
! Fonction	: Calcul des fonctions de Green au 2e ordre a partir du 1er ordre
! Auteurs	: H.Touchette (1996)
! Revision	: 20 novembre 1996 (Touchette)
! Reference	: [1] H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		      hiver 96 - ete 97.
!		  [2] H.G.Matuttis,Ph.D. Thesis,University of Regensburg,1995.
! Notes		: - G(2)=exp(-K/2)*G(1)*exp(K/2) (Multiplication a gauche et a droite)
!		  - Pour laisser les fcts de Green au 1er ordre, mettre les lignes 
!		    MAC1 et MAC2 en commentaires.
!		  
!--------------------------------------------------------------------------------------
	SUBROUTINE MACGD(NP,NM,G,GP)
	USE HOPPING,ONLY: H2P,H2M
	IMPLICIT NONE
	INTEGER,INTENT(IN)			:: NP,NM
	REAL(8),DIMENSION (NP,NP),INTENT(IN)	:: G
	REAL(8),DIMENSION (NP,NP),INTENT(OUT) 	:: GP
	CALL MAC10(NP,NM,G,GP)					! GP<=G
!	CALL MAC2(NP,NM,GP,H2P)					! GP<=GP*H2P
!	CALL MAC1(NP,NM,H2M,GP)					! GP<=H2M*GP=H2M*G*H2P
	END SUBROUTINE MACGD
!--------------------------------------------------------------------------------------
! Nom		: FTQ		[SUB]	(Fourier Transform Q-space)		MI3
! Fonction	: Transformee de Fourier des fonctions de Green a temps inegal
! Auteurs	: H.Touchette (1996)
! Revision	: 20 novembre 1996 (Touchette)
! Reference	: [1] H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		      hiver 96 - ete 97.
! Notes		: 
!--------------------------------------------------------------------------------------
	SUBROUTINE FTQ(N,M,NTRAN,G,FG)
	USE CONSTANTES,ONLY: NP,ND1,ND2,NL
	USE UTILITAIRE,ONLY: COSMAG
	IMPLICIT NONE
	REAL(8)						:: SUM
	INTEGER,INTENT(IN)				:: N,M,NTRAN
	INTEGER						:: NM,I,J	! Var locales
	INTEGER						:: KL,KX,KY	! Var locales
	REAL(8),DIMENSION(0:NL,NP,NP),INTENT(IN)	:: G
	REAL(8),DIMENSION(0:NL,ND1,ND2),INTENT(OUT)	:: FG
!
	NM=N*M
	DO KL=0,NTRAN
	   DO KX=1,N
	      DO KY=1,M
	         SUM=0.D0
	         DO I=1,NM
	            DO J=1,NM
	               SUM=SUM+COSMAG(KX,KY,I,J)*G(KL,I,J)
	            END DO
	         END DO
	         FG(KL,KX,KY)=SUM/NM
	      END DO
	   END DO
	END DO
	END SUBROUTINE FTQ
!--------------------------------------------------------------------------------------
! Nom		: SUSCEPI	[SUB]						MI4
! Fonction	: Calcul du facteur de structure de paire a temps inegal et calcul  
!		  de la susceptibilite thermodynamique
! Auteurs	: H.Touchette (1996)
! Revision	: 20 novembre 1996 (Touchette)
! Reference	: [1] H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		      hiver 96 - ete 97.
! Notes		: - Integrale sur tau de X1 en option 
!		  - X2 = SUM(0,TRAN) X(tau) (susceptibilite thermodynamique)
!--------------------------------------------------------------------------------------
	SUBROUTINE SUSCEPI(PARA,GIUP,GIDWN,X1,X2)
	USE CONSTANTES,ONLY: NP,NL,ND1,ND2
	USE UTILITAIRE,ONLY: W,COSMAG
	USE PARA_SIM
	IMPLICIT NONE
	REAL(8)						:: SUM,DTAU		! loc.
	REAL(8)						:: GPS1,GPS2		! loc.
	INTEGER						:: NM,I,J,L,KX,KY	! loc.
	REAL(8),DIMENSION (ND1,ND2),INTENT(OUT)		:: X2
	REAL(8),DIMENSION (0:NL,ND1,ND2),INTENT(OUT)	:: X1
	REAL(8),DIMENSION (0:NL,NP,NP),INTENT(IN)	:: GIUP,GIDWN
	TYPE(PARAMETRE),INTENT(IN)			:: PARA
!
	NM=PARA%N*PARA%M
	DTAU=PARA%BETA/DBLE(PARA%TRAN)
	DO L=0,PARA%TRAN
	   DO KX=1,PARA%N
	      DO KY=1,PARA%M
	         SUM=0.D0
	         DO I=1,NM
	            DO J=1,NM
	               GPS1=GIUP(L,I,J)*GIDWN(L,I,J)
	               GPS2=GIUP(-L+PARA%TRAN,J,I)*GIDWN(-L+PARA%TRAN,J,I)
		       SUM=SUM+(GPS1+GPS2)*COSMAG(KX,KY,I,J)
	            END DO
	         END DO
	         X1(L,KX,KY)=SUM/NM
	      END DO
	   END DO
	END DO
!
!	Integrale
	IF(PARA%YNMES(6).EQ.'Y'.OR.PARA%YNMES(6).EQ.'y') THEN
	   DO KX=1,PARA%N
	      DO KY=1,PARA%M
	         SUM=0.D0
	         DO L=0,PARA%TRAN
	            SUM=SUM+W(L+1)*X1(L,KX,KY)
	         END DO
	         X2(KX,KY)=SUM*DTAU
	      END DO
	   END DO
	ENDIF
	END SUBROUTINE SUSCEPI
!--------------------------------------------------------------------------------------
! Nom		: MATSUBARA	[SUB]						MI5
! Fonction	: Calcul des fonctions de Green en frequences de Matsubara
! Auteurs	: H.Touchette (1996)
! Revision	: 20 novembre 1996 (Touchette)
! Reference	: [1] H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		      hiver 96 - ete 97.
! Notes		: - G(iwn,Qx,Qy) = Re{G(iwn,Qx,Qy)} + Im{G(iwn,Qx,Qy)}
!		    Re{G(iwn,Qx,Qy)} - IST=0
!		    Im{G(iwn,Qx,Qy)} - IST=1
!		  - Integration selon la regle de Filon (cf ref)
!--------------------------------------------------------------------------------------
	SUBROUTINE MATSUBARA(N,M,NTRAN,BETA,IST,GI,GIM)
	USE CONSTANTES,ONLY: NL,ND1,ND2
	IMPLICIT NONE
	REAL(8)						:: PI,WMAT,DTAU	! Var locales
	REAL(8),INTENT(IN)				:: BETA
	INTEGER,INTENT(IN)				:: N,M,NTRAN,IST
	INTEGER						:: KX,KY,LW	! Var locales
	REAL(8),DIMENSION (0:NL,ND1,ND2),INTENT(IN)	:: GI
	REAL(8),DIMENSION (0:NL,ND1,ND2),INTENT(OUT)	:: GIM
!
	PI=DACOS(-1.D0)
	DTAU=BETA/DBLE(NTRAN)
	DO KX=1,N
	   DO KY=1,M
	      DO LW=0,NTRAN
	         WMAT=PI*DBLE(2*LW+1)/BETA
	         SELECT CASE(IST)
	            CASE(0)						
	               CALL FILON(0.D0,BETA,WMAT,NTRAN,0,GI(:,KX,KY),GIM(LW,KX,KY))
	            CASE(1)						 
	               CALL FILON(0.D0,BETA,WMAT,NTRAN,1,GI(:,KX,KY),GIM(LW,KX,KY))
	            CASE DEFAULT
	               WRITE(*,*) '--- Erreur : MATSUBARA ---'
	          END SELECT 
	      END DO
	   END DO
	END DO
	END SUBROUTINE MATSUBARA
!--------------------------------------------------------------------------------------
! Nom		: FILON		[SUB]						MI6
! Fonction	: Integration selon la regle de Filon
! Auteurs	: Liang Chen (1996)
! Revision	: 4 decembre 1996 (Touchette)
! Reference	: [1] H.Touchette, Rapports des projets Monte Carlo quantique, CRPS,
!		      hiver 96 - ete 97.
!		  [2] Davis & Rabinowitz, Methods of Numerical Integration, Academic
!		      Press, 1984. QA 299.3 D28 1984
!		  [3] Davis & Rabinowitz, Numerical Integration, Blaisdell, 1967.
!		      QA 299.3 D3N 1967
! Notes		: - IST=0 : Integrale de f(x)*cos(t*x)
!		  - IST=1 : Integrale de f(x)*sin(t*x)
!		  - Utilise un nombre impair de points: de 0 a N_EVEN 
!
!--------------------------------------------------------------------------------------
	SUBROUTINE FILON(A,B,K,N_EVEN,IST,Y,RESULT)
	USE CONSTANTES,ONLY: NL
	IMPLICIT NONE
	REAL(8)					:: H,ALPHA,BETA,GAMMA	! Var locales
	REAL(8)					:: C2N,C2N_1,KH,KA,KB	! Var locales
	REAL(8)					:: SKH,CKH,KH2,KH3	! Var locales
	REAL(8),INTENT(IN)			:: A,B,K
	REAL(8),INTENT(OUT)			:: RESULT
	INTEGER,INTENT(IN)			:: N_EVEN,IST
	INTEGER					:: I
	REAL(8),DIMENSION (0:NL),INTENT(IN)	:: Y
!
	H=(B-A)/DBLE(N_EVEN)
	KH=K*H
	KH2=KH*KH
	KH3=KH2*KH
	KA=K*A
	KB=K*B
	SKH=DSIN(KH)
	CKH=DCOS(KH)
	ALPHA=(KH2+KH*SKH*CKH-2.D0*SKH*SKH)/KH3
	BETA=2.D0*(KH*(1.D0+CKH*CKH)-2.D0*SKH*CKH)/KH3
	GAMMA=4.D0*(SKH-KH*CKH)/KH3
	SELECT CASE(IST)
	   CASE(0)
	      C2N=0.5D0*Y(0)*DCOS(KA)+0.5D0*Y(N_EVEN)*DCOS(KB)
	      DO I=2,N_EVEN-1,2
	         C2N=C2N+Y(I)*DCOS(KA+DBLE(I)*KH)
	      END DO
	      C2N_1=0.D0
	      DO I=1,N_EVEN,2
	         C2N_1=C2N_1+Y(I)*DCOS(KA+DBLE(I)*KH)
	      END DO
	      RESULT=H*(ALPHA*(Y(N_EVEN)*DSIN(KB)-Y(0)*DSIN(KA))+BETA*C2N+GAMMA*C2N_1)
	   CASE(1)
	      C2N=0.5D0*Y(0)*DSIN(KA)+0.5D0*Y(N_EVEN)*DSIN(KB)
	      DO I=2,N_EVEN-1,2
	         C2N=C2N+Y(I)*DSIN(KA+DBLE(I)*KH)
	      END DO
	      C2N_1=0.D0
	      DO I=1,N_EVEN,2
	         C2N_1=C2N_1+Y(I)*DSIN(KA+DBLE(I)*KH)
	      END DO
	      RESULT=H*(ALPHA*(-Y(N_EVEN)*DCOS(KB)+Y(0)*DCOS(KA))+BETA*C2N+GAMMA*C2N_1) 
	   CASE DEFAULT
	      WRITE(*,*) '--- Erreur : FILON ---'
	END SELECT
	END SUBROUTINE FILON
!--------------------------------------------------------------------------------------
! Nom		: SELF		[SUB]						MI7
! Fonction	: Calcul de la self-energie en frequences de Matsubara
! Auteurs	: H.Touchette
! Revision	: 20 juin 1997 (Touchette)
! References	: H.Touchette, Aspects numeriques des simulations Monte Carlo quantique 
!		  du modele de Hubbard, rapport technique du CRPS, departement de 
!		  physique, UdeS, ete 1997.
! Notes		: - SR = Re{sigma(iwn,Qx,Qy)}
!		  - SI = Im{sigma(iwn,Qx,Qy)}
!--------------------------------------------------------------------------------------
	SUBROUTINE SELF(PARA,GR,GI,SR,SI)
	USE CONSTANTES,ONLY: NL,ND1,ND2
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER						:: I,J,L
	REAL(8)						:: WMAT,PI,DEN,ENE
	REAL(8)						:: QX,QY,ZETAK
	REAL(8),DIMENSION(0:NL,ND1,ND2),INTENT(IN)	:: GR,GI
	REAL(8),DIMENSION(0:NL,ND1,ND2),INTENT(OUT)	:: SR,SI
	TYPE(PARAMETRE),INTENT(IN)			:: PARA
!
	PI=DACOS(-1.D0)
	DO I=1,PARA%N
	   DO J=1,PARA%M
	      QX=2.D0*PI*DBLE(I)/DBLE(PARA%N)
	      QY=2.D0*PI*DBLE(J)/DBLE(PARA%M)
	      ZETAK=ENE(PARA%N,PARA%M,PARA%TPAR,PARA%TPER,PARA%TPRM,QX,QY)-PARA%MU
	      DO L=0,PARA%TRAN
	         WMAT=PI*DBLE(2*L+1)/PARA%BETA
	   	 DEN=GR(L,I,J)*GR(L,I,J)+GI(L,I,J)*GI(L,I,J)
	   	 SR(L,I,J)=-ZETAK-GR(L,I,J)/DEN
	   	 SI(L,I,J)=WMAT+GI(L,I,J)/DEN
	      END DO
	   END DO
	END DO
	END SUBROUTINE SELF
!--------------------------------------------------------------------------------------
! Nom		: COVARIANCE		[SUB]					MI8
! Fonction	: Calcul de la covariance pour la methode d'entropie maximum
! Auteurs	: H.Touchette
! Revision	: 23 juillet 1997 (Touchette)
! References	: [1] H.Touchette, Aspects numeriques des simulations Monte Carlo  
!		      quantique du modele de Hubbard, rapport technique du CRPS,  
!		      departement de physique, UdeS, ete 1997.
!		  [2] Jarrel et al., Phys.Rep.,269,1996,p.145.
!		  [3] Gubernatis et al., PRB,4,12,1991,p.6011.
! Notes		: - Matrice de covariance COVM 
!		  - On calcule cette matrice seulement pour une composante de spin
!		    et a la fin d'une simulation
!--------------------------------------------------------------------------------------
	SUBROUTINE COVARIANCE(PARA,C1,COVG,XG,COVM)
	USE CONSTANTES,ONLY: NL,ND1,ND2,NCOR
	USE PARA_SIM
	IMPLICIT NONE
	INTEGER							:: L1,L2	! Loc.
	INTEGER							:: I,J,K,NMES	! Loc.
	REAL(8)							:: SUM		! Loc.
	REAL(8)							:: D1,D2	! Loc.
	REAL(8),INTENT(IN)					:: C1
	REAL(8),DIMENSION(0:NL,ND1,ND2),INTENT(IN)		:: XG
	REAL(8),DIMENSION(NCOR,0:NL,ND1,ND2),INTENT(IN)		:: COVG
	REAL(8),DIMENSION(0:NL,0:NL,ND1,ND2),INTENT(OUT)	:: COVM
	TYPE(PARAMETRE),INTENT(IN)				:: PARA
!
	NMES=INT(C1)
	DO I=1,PARA%N
	   DO J=1,PARA%M
	      DO L1=0,PARA%TRAN
	         DO L2=0,PARA%TRAN
	            SUM=0.D0
	            DO K=1,NMES
	               D1=(XG(L1,I,J)-COVG(K,L1,I,J))
	               D2=(XG(L2,I,J)-COVG(K,L2,I,J))
	               SUM=SUM+D1*D2
	            END DO
	            COVM(L1,L2,I,J)=SUM/(C1*(C1-1.D0))
	         END DO
	      END DO
	   END DO
	END DO
	END SUBROUTINE COVARIANCE
!--------------------------------------------------------------------------------------
	
